@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2000, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,13 +26,7 @@
   This file defines the NDB Cluster handler engine_condition_pushdown
 */
 
-#ifdef USE_PRAGMA_IMPLEMENTATION
-#pragma implementation				// gcc: Class implementation
-#endif
-
-#include "mysql_priv.h"
-
-#ifdef WITH_NDBCLUSTER_STORAGE_ENGINE
+#include "ha_ndbcluster_glue.h"
 #include <ndbapi/NdbApi.hpp>
 #include "ha_ndbcluster_cond.h"
 
@@ -39,7 +40,8 @@ typedef NdbDictionary::Table NDBTAB;
   position of fields that is not directly available in the Item tree.
   Also checks if condition is supported.
 */
-void ndb_serialize_cond(const Item *item, void *arg)
+static void
+ndb_serialize_cond(const Item *item, void *arg)
 {
   Ndb_cond_traverse_context *context= (Ndb_cond_traverse_context *) arg;
   DBUG_ENTER("ndb_serialize_cond");  
@@ -229,6 +231,7 @@ void ndb_serialize_cond(const Item *item, void *arg)
         context->expect(Item::STRING_ITEM);
         context->expect(Item::VARBIN_ITEM);
         context->expect(Item::FUNC_ITEM);
+        context->expect(Item::CACHE_ITEM);
         ndb_serialize_cond(rewrite_context->left_hand_item, arg);
         context->skip= 0; // Any FUNC_ITEM expression has already been parsed
         context->rewrite_stack= rewrite_context; // Enable rewrite mode
@@ -257,7 +260,7 @@ void ndb_serialize_cond(const Item *item, void *arg)
         {
           Item_field *field_item= (Item_field *) item;
           Field *field= field_item->field;
-          enum_field_types type= field->real_type();
+          const enum_field_types type= field->real_type();
           /*
             Check that the field is part of the table of the handler
             instance and that we expect a field with of this result type.
@@ -269,26 +272,31 @@ void ndb_serialize_cond(const Item *item, void *arg)
             DBUG_PRINT("info", ("table %s", tab->getName()));
             DBUG_PRINT("info", ("column %s", field->field_name));
             DBUG_PRINT("info", ("column length %u", field->field_length));
-            DBUG_PRINT("info", ("type %d", field->real_type()));
+            DBUG_PRINT("info", ("type %d", type));
             DBUG_PRINT("info", ("result type %d", field->result_type()));
 
             // Check that we are expecting a field and with the correct
             // result type and of length that can store the item value
             if (context->expecting(Item::FIELD_ITEM) &&
-                context->expecting_field_type(field->real_type()) &&
+                context->expecting_field_type(type) &&
                 context->expecting_max_length(field->field_length) &&
                 (context->expecting_field_result(field->result_type()) ||
                  // Date and year can be written as string or int
                  ((type == MYSQL_TYPE_TIME ||
+                   type == MYSQL_TYPE_TIME2 ||
                    type == MYSQL_TYPE_DATE || 
                    type == MYSQL_TYPE_NEWDATE || 
                    type == MYSQL_TYPE_YEAR ||
-                   type == MYSQL_TYPE_DATETIME)
+                   type == MYSQL_TYPE_DATETIME ||
+                   type == MYSQL_TYPE_DATETIME2)
                   ? (context->expecting_field_result(STRING_RESULT) ||
                      context->expecting_field_result(INT_RESULT))
                   : TRUE)) &&
                 // Bit fields no yet supported in scan filter
                 type != MYSQL_TYPE_BIT &&
+                /* Char(0) field is treated as Bit fields inside NDB
+                   Hence not supported in scan filter */
+                (!(type == MYSQL_TYPE_STRING && field->pack_length() == 0)) &&
                 // No BLOB support in scan filter
                 type != MYSQL_TYPE_TINY_BLOB &&
                 type != MYSQL_TYPE_MEDIUM_BLOB &&
@@ -304,10 +312,12 @@ void ndb_serialize_cond(const Item *item, void *arg)
               {
                 // We have not seen second argument yet
                 if (type == MYSQL_TYPE_TIME ||
+                    type == MYSQL_TYPE_TIME2 ||
                     type == MYSQL_TYPE_DATE || 
                     type == MYSQL_TYPE_NEWDATE || 
                     type == MYSQL_TYPE_YEAR ||
-                    type == MYSQL_TYPE_DATETIME)
+                    type == MYSQL_TYPE_DATETIME ||
+                    type == MYSQL_TYPE_DATETIME2)
                 {
                   context->expect_only(Item::STRING_ITEM);
                   context->expect(Item::INT_ITEM);
@@ -348,10 +358,12 @@ void ndb_serialize_cond(const Item *item, void *arg)
                 if ((field->result_type() == STRING_RESULT) &&
                     !context->expecting_collation(item->collation.collation)
                     && type != MYSQL_TYPE_TIME
+                    && type != MYSQL_TYPE_TIME2
                     && type != MYSQL_TYPE_DATE
                     && type != MYSQL_TYPE_NEWDATE
                     && type != MYSQL_TYPE_YEAR
-                    && type != MYSQL_TYPE_DATETIME)
+                    && type != MYSQL_TYPE_DATETIME
+                    && type != MYSQL_TYPE_DATETIME2)
                 {
                   DBUG_PRINT("info", ("Found non-matching collation %s",  
                                       item->collation.collation->name)); 
@@ -497,9 +509,15 @@ void ndb_serialize_cond(const Item *item, void *arg)
           case Item_func::LIKE_FUNC:
           {
             Ndb_expect_stack* expect_next= new Ndb_expect_stack();
-            DBUG_PRINT("info", ("LIKE_FUNC"));      
+            DBUG_PRINT("info", ("LIKE_FUNC"));
+
+            if (((const Item_func_like *)func_item)->escape_was_used_in_parsing())
+            {
+              DBUG_PRINT("info", ("LIKE expressions with ESCAPE not supported"));
+              context->supported= FALSE;
+            }
             curr_cond->ndb_item= new Ndb_item(func_item->functype(),
-                                              func_item);      
+                                              func_item);
 
             /*
               Ndb currently only supports pushing
@@ -578,6 +596,7 @@ void ndb_serialize_cond(const Item *item, void *arg)
             context->expect(Item::STRING_ITEM);
             context->expect(Item::VARBIN_ITEM);
             context->expect(Item::FUNC_ITEM);
+            context->expect(Item::CACHE_ITEM);
             break;
           }
           case Item_func::IN_FUNC:
@@ -605,6 +624,7 @@ void ndb_serialize_cond(const Item *item, void *arg)
             context->expect(Item::STRING_ITEM);
             context->expect(Item::VARBIN_ITEM);
             context->expect(Item::FUNC_ITEM);
+            context->expect(Item::CACHE_ITEM);
             break;
           }
           case Item_func::NEG_FUNC:
@@ -758,11 +778,9 @@ void ndb_serialize_cond(const Item *item, void *arg)
           {
 #ifndef DBUG_OFF
             char buff[256];
-            String str(buff,(uint32) sizeof(buff), system_charset_info);
-            str.length(0);
-            Item_string *string_item= (Item_string *) item;
-            DBUG_PRINT("info", ("value \"%s\"", 
-                                string_item->val_str(&str)->ptr()));
+            String str(buff, 0, system_charset_info);
+            const_cast<Item*>(item)->print(&str, QT_ORDINARY);
+            DBUG_PRINT("info", ("value: '%s'", str.c_ptr_safe()));
 #endif
             NDB_ITEM_QUALIFICATION q;
             q.value_type= Item::STRING_ITEM;
@@ -923,9 +941,149 @@ void ndb_serialize_cond(const Item *item, void *arg)
           }
           break;
         }
+        case Item::CACHE_ITEM:
+        {
+          DBUG_PRINT("info", ("CACHE_ITEM"));
+          Item_cache* cache_item = (Item_cache*)item;
+          DBUG_PRINT("info", ("result type %d", cache_item->result_type()));
+
+          // Item_cache has cached "something", use its value
+          // based on the result_type of the item
+          switch(cache_item->result_type())
+          {
+          case INT_RESULT:
+            DBUG_PRINT("info", ("INT_RESULT"));
+            if (context->expecting(Item::INT_ITEM)) 
+            {
+              DBUG_PRINT("info", ("value %ld",
+                                  (long) ((Item_int*) item)->value));
+              NDB_ITEM_QUALIFICATION q;
+              q.value_type= Item::INT_ITEM;
+              curr_cond->ndb_item= new Ndb_item(NDB_VALUE, q, item);
+              if (! context->expecting_no_field_result()) 
+              {
+                // We have not seen the field argument yet
+                context->expect_only(Item::FIELD_ITEM);
+                context->expect_only_field_result(INT_RESULT);
+                context->expect_field_result(REAL_RESULT);
+                context->expect_field_result(DECIMAL_RESULT);
+              }
+              else
+              {
+                // Expect another logical expression
+                context->expect_only(Item::FUNC_ITEM);
+                context->expect(Item::COND_ITEM);
+              }
+            }
+            else
+              context->supported= FALSE;
+            break;
+
+          case REAL_RESULT:
+            DBUG_PRINT("info", ("REAL_RESULT"));
+            if (context->expecting(Item::REAL_ITEM)) 
+            {
+              DBUG_PRINT("info", ("value %f", ((Item_float*) item)->value));
+              NDB_ITEM_QUALIFICATION q;
+              q.value_type= Item::REAL_ITEM;
+              curr_cond->ndb_item= new Ndb_item(NDB_VALUE, q, item);
+              if (! context->expecting_no_field_result()) 
+              {
+                // We have not seen the field argument yet
+                context->expect_only(Item::FIELD_ITEM);
+                context->expect_only_field_result(REAL_RESULT);
+              }
+              else
+              {
+                // Expect another logical expression
+                context->expect_only(Item::FUNC_ITEM);
+                context->expect(Item::COND_ITEM);
+              }
+            }
+            else
+              context->supported= FALSE;
+            break;
+
+          case DECIMAL_RESULT:
+            DBUG_PRINT("info", ("DECIMAL_RESULT"));
+            if (context->expecting(Item::DECIMAL_ITEM)) 
+            {
+              DBUG_PRINT("info", ("value %f",
+                                  ((Item_decimal*) item)->val_real()));
+              NDB_ITEM_QUALIFICATION q;
+              q.value_type= Item::DECIMAL_ITEM;
+              curr_cond->ndb_item= new Ndb_item(NDB_VALUE, q, item);
+              if (! context->expecting_no_field_result()) 
+              {
+                // We have not seen the field argument yet
+                context->expect_only(Item::FIELD_ITEM);
+                context->expect_only_field_result(REAL_RESULT);
+                context->expect_field_result(DECIMAL_RESULT);
+              }
+              else
+              {
+                // Expect another logical expression
+                context->expect_only(Item::FUNC_ITEM);
+                context->expect(Item::COND_ITEM);
+              }
+            }
+            else
+              context->supported= FALSE;
+            break;
+
+          case STRING_RESULT:
+            DBUG_PRINT("info", ("STRING_RESULT")); 
+            // Check that we do support pushing the item value length
+            if (context->expecting(Item::STRING_ITEM) &&
+                context->expecting_length(item->max_length)) 
+            {
+  #ifndef DBUG_OFF
+              char buff[256];
+              String str(buff, 0, system_charset_info);
+              const_cast<Item*>(item)->print(&str, QT_ORDINARY);
+              DBUG_PRINT("info", ("value: '%s'", str.c_ptr_safe()));
+  #endif
+              NDB_ITEM_QUALIFICATION q;
+              q.value_type= Item::STRING_ITEM;
+              curr_cond->ndb_item= new Ndb_item(NDB_VALUE, q, item);      
+              if (! context->expecting_no_field_result())
+              {
+                // We have not seen the field argument yet
+                context->expect_only(Item::FIELD_ITEM);
+                context->expect_only_field_result(STRING_RESULT);
+                context->expect_collation(item->collation.collation);
+                context->expect_length(item->max_length);
+              }
+              else 
+              {
+                // Expect another logical expression
+                context->expect_only(Item::FUNC_ITEM);
+                context->expect(Item::COND_ITEM);
+                context->expect_no_length();
+                // Check that we are comparing with a field with same collation
+                if (!context->expecting_collation(item->collation.collation))
+                {
+                  DBUG_PRINT("info", ("Found non-matching collation %s",  
+                                      item->collation.collation->name));
+                  context->supported= FALSE;
+                }
+              }
+            }
+            else
+              context->supported= FALSE;
+            break;
+
+          default:
+            context->supported= FALSE;
+            break;
+          }
+          break;
+        }
+
         default:
         {
-          DBUG_PRINT("info", ("Found item of type %d", item->type()));
+          DBUG_PRINT("info", ("Found unsupported item of type %d",
+                              item->type()));
           context->supported= FALSE;
         }
         }
@@ -962,16 +1120,16 @@ void ndb_serialize_cond(const Item *item, void *arg)
   Push a condition
  */
 const 
-COND* 
-ha_ndbcluster_cond::cond_push(const COND *cond, 
+Item* 
+ha_ndbcluster_cond::cond_push(const Item *cond, 
                               TABLE *table, const NDBTAB *ndb_table)
 { 
-  DBUG_ENTER("cond_push");
+  DBUG_ENTER("ha_ndbcluster_cond::cond_push");
   Ndb_cond_stack *ndb_cond = new Ndb_cond_stack();
   if (ndb_cond == NULL)
   {
     my_errno= HA_ERR_OUT_OF_MEM;
-    DBUG_RETURN(NULL);
+    DBUG_RETURN(cond);
   }
   if (m_cond_stack)
     ndb_cond->next= m_cond_stack;
@@ -1019,8 +1177,9 @@ ha_ndbcluster_cond::cond_clear()
 }
 
 bool
-ha_ndbcluster_cond::serialize_cond(const COND *cond, Ndb_cond_stack *ndb_cond,
-                                   TABLE *table, const NDBTAB *ndb_table)
+ha_ndbcluster_cond::serialize_cond(const Item *cond, Ndb_cond_stack *ndb_cond,
+                                   TABLE *table,
+                                   const NDBTAB *ndb_table) const
 {
   DBUG_ENTER("serialize_cond");
   Item *item= (Item *) cond;
@@ -1037,7 +1196,7 @@ ha_ndbcluster_cond::serialize_cond(const COND *cond, Ndb_cond_stack *ndb_cond,
 int
 ha_ndbcluster_cond::build_scan_filter_predicate(Ndb_cond * &cond, 
                                                 NdbScanFilter *filter,
-                                                bool negated)
+                                                bool negated) const
 {
   DBUG_ENTER("build_scan_filter_predicate");  
   switch (cond->ndb_item->type) {
@@ -1209,18 +1368,19 @@ ha_ndbcluster_cond::build_scan_filter_predicate(Ndb_cond * &cond,
     case NDB_LIKE_FUNC:
     {
       if (!value || !field) break;
-      if ((value->qualification.value_type != Item::STRING_ITEM) &&
-          (value->qualification.value_type != Item::VARBIN_ITEM))
-          break;
-      // Save value in right format for the field type
-      value->save_in_field(field);
-      DBUG_PRINT("info", ("Generating LIKE filter: like(%d,%s,%d)", 
-                          field->get_field_no(), value->get_val(), 
-                          value->pack_length()));
-      if (filter->cmp(NdbScanFilter::COND_LIKE, 
+      DBUG_ASSERT(field == a && value == b);
+      char buff[MAX_FIELD_WIDTH];
+      String str(buff, sizeof(buff), field->get_field_charset());
+      Item *value_item = const_cast<Item *>(value->get_item());
+      const String *pattern = value_item->val_str(&str);
+      DBUG_PRINT("info", ("Generating LIKE filter: like(%d,%s,%u)",
+                          field->get_field_no(),
+                          pattern->ptr(),
+                          pattern->length()));
+      if (filter->cmp(NdbScanFilter::COND_LIKE,
                       field->get_field_no(),
-                      value->get_val(),
-                      value->pack_length()) == -1)
+                      pattern->ptr(),
+                      pattern->length()) == -1)
         DBUG_RETURN(1);
       cond= cond->next->next->next;
       DBUG_RETURN(0);
@@ -1228,18 +1388,19 @@ ha_ndbcluster_cond::build_scan_filter_predicate(Ndb_cond * &cond,
     case NDB_NOTLIKE_FUNC:
     {
       if (!value || !field) break;
-      if ((value->qualification.value_type != Item::STRING_ITEM) &&
-          (value->qualification.value_type != Item::VARBIN_ITEM))
-          break;
-      // Save value in right format for the field type
-      value->save_in_field(field);
-      DBUG_PRINT("info", ("Generating NOTLIKE filter: notlike(%d,%s,%d)", 
-                          field->get_field_no(), value->get_val(), 
-                          value->pack_length()));
-      if (filter->cmp(NdbScanFilter::COND_NOT_LIKE, 
+      DBUG_ASSERT(field == a && value == b);
+      char buff[MAX_FIELD_WIDTH];
+      String str(buff, sizeof(buff), field->get_field_charset());
+      Item *value_item = const_cast<Item *>(value->get_item());
+      const String *pattern = value_item->val_str(&str);
+      DBUG_PRINT("info", ("Generating NOTLIKE filter: notlike(%d,%s,%u)",
+                          field->get_field_no(),
+                          pattern->ptr(),
+                          pattern->length()));
+      if (filter->cmp(NdbScanFilter::COND_NOT_LIKE,
                       field->get_field_no(),
-                      value->get_val(),
-                      value->pack_length()) == -1)
+                      pattern->ptr(),
+                      pattern->length()) == -1)
         DBUG_RETURN(1);
       cond= cond->next->next->next;
       DBUG_RETURN(0);
@@ -1277,7 +1438,7 @@ ha_ndbcluster_cond::build_scan_filter_predicate(Ndb_cond * &cond,
 
 int
 ha_ndbcluster_cond::build_scan_filter_group(Ndb_cond* &cond, 
-                                            NdbScanFilter *filter)
+                                            NdbScanFilter *filter) const
 {
   uint level=0;
   bool negated= FALSE;
@@ -1351,7 +1512,8 @@ ha_ndbcluster_cond::build_scan_filter_group(Ndb_cond* &cond,
 
 
 int
-ha_ndbcluster_cond::build_scan_filter(Ndb_cond * &cond, NdbScanFilter *filter)
+ha_ndbcluster_cond::build_scan_filter(Ndb_cond * &cond,
+                                      NdbScanFilter *filter) const
 {
   bool simple_cond= TRUE;
   DBUG_ENTER("build_scan_filter");  
@@ -1382,7 +1544,7 @@ ha_ndbcluster_cond::build_scan_filter(Ndb_cond * &cond, NdbScanFilter *filter)
 
 int
 ha_ndbcluster_cond::generate_scan_filter(NdbInterpretedCode* code,
-                                         NdbScanOperation::ScanOptions* options)
+                                         NdbScanOperation::ScanOptions* options) const
 {
   DBUG_ENTER("generate_scan_filter");
 
@@ -1398,13 +1560,13 @@ ha_ndbcluster_cond::generate_scan_filter(NdbInterpretedCode* code,
       {
         // err.message has static storage
         DBUG_PRINT("info", ("%s", err.message));
-        push_warning(current_thd, MYSQL_ERROR::WARN_LEVEL_WARN,
+        push_warning(current_thd, Sql_condition::WARN_LEVEL_WARN,
                      err.code, err.message);
       }
       else
         DBUG_RETURN(ret);
     }
-    else
+    else if (options!=NULL)
     {
       options->interpretedCode= code;
       options->optionsPresent|= NdbScanOperation::ScanOptions::SO_INTERPRETED;
@@ -1420,7 +1582,7 @@ ha_ndbcluster_cond::generate_scan_filter(NdbInterpretedCode* code,
 
 
 int
-ha_ndbcluster_cond::generate_scan_filter_from_cond(NdbScanFilter& filter)
+ha_ndbcluster_cond::generate_scan_filter_from_cond(NdbScanFilter& filter) const
 {
   bool multiple_cond= FALSE;
   DBUG_ENTER("generate_scan_filter_from_cond");
@@ -1462,15 +1624,14 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
                                                       NdbScanOperation::ScanOptions* options,
                                                       const KEY* key_info, 
                                                       const key_range *start_key,
-                                                      const key_range *end_key,
-                                                      uchar *buf)
+                                                      const key_range *end_key) const
 {
   DBUG_ENTER("generate_scan_filter_from_key");
 
 #ifndef DBUG_OFF
   {
     DBUG_PRINT("info", ("key parts:%u length:%u",
-                        key_info->key_parts, key_info->key_length));
+                        key_info->user_defined_key_parts, key_info->key_length));
     const key_range* keylist[2]={ start_key, end_key };
     for (uint j=0; j <= 1; j++)
     {
@@ -1485,7 +1646,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
         sprintf(buf, "key range %u: flag:%u part", j, key->flag);
         const KEY_PART_INFO* key_part=key_info->key_part;
         const uchar* ptr=key->key;
-        for (uint i=0; i < key_info->key_parts; i++)
+        for (uint i=0; i < key_info->user_defined_key_parts; i++)
         {
           sprintf(buf+strlen(buf), " %u:", i);
           for (uint k=0; k < key_part->store_length; k++)
@@ -1524,7 +1685,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
     if (start_key != 0 &&
         start_key->flag == HA_READ_AFTER_KEY &&
         end_key == 0 &&
-        key_info->key_parts == 1)
+        key_info->user_defined_key_parts == 1)
     {
       const KEY_PART_INFO* key_part=key_info->key_part;
       if (key_part->null_bit != 0) // nullable (must be)
@@ -1558,7 +1719,7 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
     {
       const KEY_PART_INFO* key_part=key_info->key_part;
       const uchar* ptr=start_key->key;
-      for (uint i=0; i < key_info->key_parts; i++)
+      for (uint i=0; i < key_info->user_defined_key_parts; i++)
       {
         const Field* field=key_part->field;
         if (key_part->null_bit) // nullable
@@ -1614,11 +1775,12 @@ int ha_ndbcluster_cond::generate_scan_filter_from_key(NdbInterpretedCode* code,
     
   if (filter.end() == -1)
     DBUG_RETURN(1);
-
-  options->interpretedCode= code;
-  options->optionsPresent|= NdbScanOperation::ScanOptions::SO_INTERPRETED;
+  
+  if (options!=NULL)
+  {
+    options->interpretedCode= code;
+    options->optionsPresent|= NdbScanOperation::ScanOptions::SO_INTERPRETED;
+  }
 
   DBUG_RETURN(0);
 }
-
-#endif /* HAVE_NDBCLUSTER_DB */
