@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -133,18 +140,22 @@ class NdbRecordScanResultDataImpl extends NdbRecordResultDataImpl {
                                 recordsLocked.add(lockedRecord);
                             }
                         }
+                        // check the NdbRecord buffer guard
                         // load blob data into the operation
                         scanOperation.loadBlobValues();
                         return true;
                     } else {
                         // skip this record
+                        scanOperation.returnValueBuffer();
                         break;
                     }
                 case SCAN_FINISHED:
+                    scanOperation.returnValueBuffer();
                     executeIfRecordsLocked();
-                    ndbScanOperation.close(true, true);
+                    scanOperation.close();
                     return false;
                 case CACHE_EMPTY:
+                    scanOperation.returnValueBuffer();
                     executeIfRecordsLocked();
                     fetch = true;
                     break;
