@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -21,6 +27,9 @@
 
 #include "SignalData.hpp"
 
+#define JAM_FILE_ID 163
+
+
 /**
  * This is signal is sent from TC to API
  * It means that the transaction was committed
@@ -37,6 +46,8 @@ class TcCommitConf {
   friend class Ndb;
   friend class NdbTransaction;
 
+  friend class TransporterFacade;
+
 public:
   STATIC_CONST( SignalLength = 5 );
 private:
@@ -76,4 +87,7 @@ private:
   Uint32 errorCode;
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
