@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -1056,7 +1063,8 @@ public:
                  OO_LOCKHANDLE   = 0x80,
                  OO_QUEUABLE     = 0x100,
                  OO_NOT_QUEUABLE = 0x200,
-                 OO_DEFERRED_CONSTAINTS = 0x400
+                 OO_DEFERRED_CONSTAINTS = 0x400,
+                 OO_DISABLE_FK   = 0x800
     };
 
     /* An operation-specific abort option.
@@ -1099,6 +1107,11 @@ public:
   const NdbLockHandle* getLockHandle() const;
   const NdbLockHandle* getLockHandle();
 
+#ifndef DOXYGEN_SHOULD_SKIP_INTERNAL
+  // XXX until NdbRecord is used in ndb_restore
+  void set_disable_fk() { m_flags |= OF_DISABLE_FK; }
+#endif
+
 protected:
 /******************************************************************************
  * These are the methods used to create and delete the NdbOperation objects.
@@ -1111,7 +1124,7 @@ protected:
 //--------------------------------------------------------------
 // Initialise after allocating operation to a transaction		      
 //--------------------------------------------------------------
-  int init(const class NdbTableImpl*, NdbTransaction* aCon, bool useRec);
+  int init(const class NdbTableImpl*, NdbTransaction* aCon);
   void initInterpreter();
 
   NdbOperation(Ndb* aNdb, Type aType = PrimaryKeyAccess);	
@@ -1456,7 +1469,8 @@ protected:
     */
     OF_USE_ANY_VALUE = 0x2,
     OF_QUEUEABLE = 0x4,
-    OF_DEFERRED_CONSTRAINTS = 0x8
+    OF_DEFERRED_CONSTRAINTS = 0x8,
+    OF_DISABLE_FK = 0x10
   };
   Uint8  m_flags;
 
@@ -1626,7 +1640,7 @@ inline
 const NdbRecAttr*
 NdbOperation::getFirstRecAttr() const 
 {
-  return theReceiver.theFirstRecAttr;
+  return theReceiver.m_firstRecAttr;
 }
 
 /******************************************************************************
