@@ -1,19 +1,26 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef DBDIH_H
 #define DBDIH_H
@@ -90,12 +97,6 @@
 #define ZWRONG_START_NODE_ERROR 303
 #define ZNO_REPLICA_FOUND_ERROR 304
 
-// --------------------------------------
-// Codes from LQH
-// --------------------------------------
-#define ZNODE_FAILURE_ERROR 400
-
-
 /*#########*/
 /* PHASES  */
 /*#########*/
@@ -1064,7 +1065,7 @@ private:
   void tableUpdateLab(Signal *, TabRecordPtr regTabPtr);
   void checkLcpCompletedLab(Signal *);
   void initLcpLab(Signal *, Uint32 masterRef, Uint32 tableId);
-  void startGcpLab(Signal *, Uint32 aWaitTime);
+  void startGcpLab(Signal *);
   void checkGcpStopLab(Signal *);
   void MASTER_GCPhandling(Signal *, Uint32 failedNodeId);
   void MASTER_LCPhandling(Signal *, Uint32 failedNodeId);
@@ -1117,6 +1118,8 @@ private:
   void failedNodeSynchHandling(Signal *, NodeRecordPtr failedNodePtr);
   void checkCopyTab(Signal*, NodeRecordPtr failedNodePtr);
 
+  Uint32 compute_max_failure_time();
+  void setGCPStopTimeouts();
   void initCommonData();
   void initialiseRecordsLab(Signal *, Uint32 stepNo, Uint32, Uint32);
 
@@ -1553,6 +1556,8 @@ private:
     Uint32 lcpStopGcp; 
     Uint32 keepGci;      /* USED TO CALCULATE THE GCI TO KEEP AFTER A LCP  */
     Uint32 oldestRestorableGci;
+
+    bool lcpManualStallStart; /* User requested stall of start (testing only) */
     
     NDB_TICKS m_start_time; // When last LCP was started
     Uint64    m_lcp_time;   // How long last LCP took
@@ -1787,6 +1792,17 @@ private:
     WaitGCPMasterRecord() { clientRef = 0;}
     Uint32 clientData;
     BlockReference clientRef;
+    /**
+     * GCI which must be completed before CONF sent
+     * For WaitEpoch, it is not used, the next
+     * completing epoch sends a CONF.
+     */
+    Uint32 waitGCI;
+
+    /**
+     * Special value indicating a request for shutdown sync
+     */
+    static const Uint32 ShutdownSyncGci = 0xffffffff;
 
     union { Uint32 nextPool; Uint32 nextList; };
     Uint32 prevList;
@@ -1809,8 +1825,13 @@ private:
 
   void checkWaitGCPProxy(Signal*, NodeId failedNodeId);
   void checkWaitGCPMaster(Signal*, NodeId failedNodeId);
+  void checkShutdownSync();
   void emptyWaitGCPMasterQueue(Signal*, Uint64, WaitGCPList&);
-  
+
+  void getNodeBitmap(NdbNodeBitmask& map,
+                     Uint32 listHead,
+                     int (*versionFunction) (Uint32));
+
   /**
    * Stop me
    */
@@ -1957,6 +1978,17 @@ private:
   Uint32 getMinVersion() const;
 
   bool c_2pass_inr;
+
+  void isolateNodes(Signal* signal,
+                    Uint32 delayMillis,
+                    const NdbNodeBitmask& victims);
+
+  NodeId c_handled_master_take_over_copy_gci;
+
+  bool handle_master_take_over_copy_gci(Signal *signal,
+                                        NodeId newMasterNodeId);
+
+  NdbNodeBitmask c_shutdownReqNodes;
 };
 
 #if (DIH_CDATA_SIZE < _SYSFILE_SIZE32)
