@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2000, 2012, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2012, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -22,33 +29,22 @@
 #include <signaldata/LqhKey.hpp>
 #include <signaldata/ScanFrag.hpp>
 #include <AttributeHeader.hpp>
-#include <SLFifoList.hpp>
-#include <DLFifoList.hpp>
-#include <SLList.hpp>
+#include <IntrusiveList.hpp>
 #include <ArenaPool.hpp>
 #include <DataBuffer2.hpp>
 #include <Bitmask.hpp>
 #include <signaldata/DbspjErr.hpp>
 #include "../dbtup/tuppage.hpp"
 
+#define JAM_FILE_ID 481
+
+
 class SectionReader;
 struct QueryNode;
 struct QueryNodeParameters;
 
 //#define SPJ_TRACE_TIME
 
-#ifdef SPJ_TRACE_TIME
-static
-inline
-Uint64 spj_now()
-{
-  NDB_TICKS sec;
-  Uint32 micro;
-  NdbTick_CurrentMicrosecond(&sec, &micro);
-  return Uint64(sec * 1000000 + micro);
-}
-#endif
-
 class Dbspj: public SimulatedBlock {
 public:
   Dbspj(Block_context& ctx, Uint32 instanceNumber = 0);
@@ -112,6 +108,8 @@ public:
   struct Request;
   struct TreeNode;
   struct ScanFragHandle;
+  typedef DataBuffer2<14, LocalArenaPoolImpl> Correlation_list;
+  typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_correlation_list;
   typedef DataBuffer2<14, LocalArenaPoolImpl> Dependency_map;
   typedef LocalDataBuffer2<14, LocalArenaPoolImpl> Local_dependency_map;
   typedef DataBuffer2<14, LocalArenaPoolImpl> PatternStore;
@@ -152,6 +150,12 @@ public:
   };
   typedef Ptr<TableRecord> TableRecordPtr;
 
+  enum Buffer_type {
+    BUFFER_VOID  = 0,
+    BUFFER_STACK = 1,
+    BUFFER_VAR   = 2
+  };
+
   struct RowRef
   {
     Uint32 m_page_id;
@@ -159,7 +163,7 @@ public:
     union
     {
       Uint16 unused;
-      Uint16 m_allocator;
+      enum Buffer_type m_alloc_type:16;
     };
 
     void copyto_link(Uint32 * dst) const {
@@ -198,7 +202,6 @@ public:
   struct RowPtr
   {
     Uint32 m_type;
-    Uint32 m_src_node_no;
     Uint32 m_src_node_ptrI;
     Uint32 m_src_correlation;
 
@@ -234,8 +237,26 @@ public:
     };
   };
 
-  struct SLFifoRowList
+  struct RowBuffer;  // forward decl.
+
+  /**
+   * Define overlayed 'base class' for SLFifoRowList and RowMap.
+   * As we want these to be POD struct, we does not use 
+   * inheritance, but have to take care that first part
+   * of these struct are correctly overlayed.
+   */
+  struct RowCollectionBase
   {
+    RowBuffer* m_rowBuffer;
+  };
+
+  struct SLFifoRowList //: public RowCollectionBase
+  {
+    /**
+     * BEWARE: Overlayed 'struct RowCollectionBase'
+     */
+    RowBuffer* m_rowBuffer;
+
     /**
      * Data used for a single linked list of rows
      */
@@ -244,12 +265,21 @@ public:
     Uint16 m_first_row_page_pos;
     Uint16 m_last_row_page_pos;
 
+    void construct(RowBuffer& rowBuffer) {
+      m_rowBuffer = &rowBuffer;
+      init();
+    }
     void init() { m_first_row_page_id = RNIL;}
     bool isNull() const { return m_first_row_page_id == RNIL; }
   };
 
-  struct RowMap
+  struct RowMap //: public RowCollectionBase
   {
+    /**
+     * BEWARE: Overlayed 'struct RowCollectionBase'
+     */
+    RowBuffer* m_rowBuffer;
+
     /**
      * Data used for a map with rows (key is correlation id)
      *   currently a single array is used to store row references
@@ -259,7 +289,18 @@ public:
     Uint16 m_size;                // size of array
     Uint16 m_elements;            // #elements in array
 
-    void init() { m_map_ref.setNull();}
+    void construct(RowBuffer& rowBuffer,
+                   Uint32 capacity)
+    {
+      m_rowBuffer = &rowBuffer;
+      m_size = capacity;
+      init();
+    }
+    void init() {
+      m_map_ref.setNull();
+      m_elements = 0;
+    }
+
     bool isNull() const { return m_map_ref.isNull(); }
 
     void assign (RowRef ref) {
@@ -296,7 +337,14 @@ public:
     STATIC_CONST( MAP_SIZE_PER_REF_16 = 3 );
   };
 
-  struct SLFifoRowListIterator
+  /**
+   * Define overlayed 'base class' for SLFifoRowListIterator
+   * and RowMapIterator.
+   * As we want these to be POD struct, we does not use 
+   * inheritance, but have to take care that first part
+   * of these struct are correctly overlayed.
+   */
+  struct RowIteratorBase
   {
     RowRef m_ref;
     Uint32 * m_row_ptr;
@@ -305,27 +353,96 @@ public:
     void setNull() { m_ref.setNull(); }
   };
 
-  struct SLFifoRowListIteratorPtr
+  struct SLFifoRowListIterator //: public RowIteratorBase
   {
+    /**
+     * BEWARE: Overlayed 'struct RowIteratorBase'
+     */
     RowRef m_ref;
+    Uint32 * m_row_ptr;
+
+    bool isNull() const { return m_ref.isNull(); }
+    void setNull() { m_ref.setNull(); }
+    // END: RowIteratorBase
   };
 
-  struct RowMapIterator
+  struct RowMapIterator //: public RowIteratorBase
   {
+    /**
+     * BEWARE: Overlayed 'struct RowIteratorBase'
+     */
+    RowRef m_ref;
     Uint32 * m_row_ptr;
+
+    bool isNull() const { return m_ref.isNull(); }
+    void setNull() { m_ref.setNull(); }
+    // END: RowIteratorBase
+
     Uint32 * m_map_ptr;
-    RowRef m_ref; // position of actual row
     Uint16 m_size;
     Uint16 m_element_no;
-    bool isNull() const { return m_ref.isNull(); }
-    void setNull() { m_ref.setNull(); }
   };
 
-  struct RowMapIteratorPtr
+  /**
+   * Abstraction of SLFifoRowList & RowMap
+   */
+  struct RowCollection
   {
-    Uint32 m_element_no;
+    enum collection_type
+    {
+      COLLECTION_VOID,
+      COLLECTION_MAP,
+      COLLECTION_LIST
+    };
+    union
+    {
+      RowCollectionBase m_base;  // Common part for map & list
+      SLFifoRowList m_list;
+      RowMap m_map;
+    };
+
+    RowCollection() : m_type(COLLECTION_VOID) {}
+
+    void construct(collection_type type,
+                   RowBuffer& rowBuffer,
+                   Uint32 capacity)
+    {
+      m_type = type;
+      if (m_type == COLLECTION_MAP)
+        m_map.construct(rowBuffer,capacity);
+      else if (m_type == COLLECTION_LIST)
+        m_list.construct(rowBuffer);
+    }
+
+    void init() {
+      if (m_type == COLLECTION_MAP)
+        m_map.init();
+      else if (m_type == COLLECTION_LIST)
+        m_list.init();
+    }
+
+    Uint32 rowOffset() const {
+      return (m_type == COLLECTION_MAP) ? 0 : 2;
+    }
+
+    collection_type m_type;
   };
 
+  struct RowIterator
+  {
+    union
+    {
+      RowIteratorBase m_base;  // Common part for map & list
+      SLFifoRowListIterator m_list;
+      RowMapIterator m_map;
+    };
+    RowCollection::collection_type m_type;
+
+    RowIterator() { init(); }
+    void init() { m_base.setNull(); }
+    bool isNull() const { return m_base.isNull(); }
+  };
+ 
 
   /**
    * A struct used when building an TreeNode
@@ -368,11 +485,24 @@ public:
 
   struct RowBuffer
   {
-    RowBuffer() { stack_init(); }
+    enum Buffer_type m_type;
+
+    RowBuffer() : m_type(BUFFER_VOID) {}
     DLFifoList<RowPage>::Head m_page_list;
 
-    void stack_init() { new (&m_page_list) DLFifoList<RowPage>::Head(); m_stack.m_pos = 0xFFFF; }
-    void var_init() { new (&m_page_list) DLFifoList<RowPage>::Head(); m_var.m_free = 0; }
+    void init(enum Buffer_type type)
+    {
+      new (&m_page_list) DLFifoList<RowPage>::Head();
+      m_type = type;
+      reset();
+    }
+    void reset()
+    {
+      if (m_type == BUFFER_STACK)
+        m_stack.m_pos = 0xFFFF;
+      else if (m_type == BUFFER_VAR)
+        m_var.m_free = 0;
+    }
 
     struct Stack
     {
@@ -501,6 +631,20 @@ public:
      *  should only do local cleanup(s)
      */
     void (Dbspj::*m_cleanup)(Ptr<Request>, Ptr<TreeNode>);
+
+    /**
+     * This function is called to check the node validity within
+     * the request during debug execution
+     */
+    bool (Dbspj::*m_checkNode)(const Ptr<Request>, const Ptr<TreeNode>);
+
+    /**
+     * This function is called to dump request info to the node
+     * log for debugging purposes.  It should be used for
+     * treenode type-specific data, the generic treenode info
+     * is handled by dumpNodeCommon()
+     */
+    void (Dbspj::*m_dumpNode)(const Ptr<Request>, const Ptr<TreeNode>);
   };  //struct OpInfo
 
   struct LookupData
@@ -513,11 +657,6 @@ public:
      * all rows for this operation in this batch.
      */
     Uint32 m_outstanding;
-    /**
-     * If true, the parent operation has received all the rows it will get
-     * in this batch.
-     */
-    bool m_parent_batch_complete;
     Uint32 m_lqhKeyReq[LqhKeyReq::FixedSignalLength + 4];
   };
 
@@ -560,7 +699,7 @@ public:
     }
     struct RangeBuilder
     {
-      Uint16 m_range_size;
+      Uint32 m_range_size;
       Uint16 m_range_cnt; // too set bounds info correctly
     } m_range_builder;
     Uint32 m_rangePtrI;
@@ -660,6 +799,28 @@ public:
     Uint32 m_scanFragReq[ScanFragReq::SignalLength + 2];
   };
 
+  struct DeferredParentOps
+  {
+    /**
+     * m_correlations contains a list of Correlation Values (Uint32)
+     * which identifies parent rows which has been deferred. 
+     * m_pos are index into this array, identifying the next parent row
+     * for which to resume operation.
+     */
+    Correlation_list::Head m_correlations;
+    Uint16 m_pos; // Next row operation to resume 
+
+    DeferredParentOps() : m_correlations(), m_pos(0) {}
+
+    void init()  {
+      m_correlations.init();
+      m_pos = 0;
+    }
+    bool isEmpty() const {
+      return (m_pos == m_correlations.getSize());
+    }
+  };
+
   struct TreeNode_cursor_ptr
   {
     Uint32 nextList;
@@ -677,7 +838,8 @@ public:
     TreeNode()
     : m_magic(MAGIC), m_state(TN_END),
       m_parentPtrI(RNIL), m_requestPtrI(RNIL),
-      m_ancestors()
+      m_ancestors(),
+      m_resumeEvents(0), m_resumePtrI(RNIL)
     {
     }
 
@@ -686,6 +848,7 @@ public:
       m_info(0), m_bits(T_LEAF), m_state(TN_BUILDING),
       m_parentPtrI(RNIL), m_requestPtrI(request),
       m_ancestors(),
+      m_resumeEvents(0), m_resumePtrI(RNIL),
       nextList(RNIL), prevList(RNIL)
     {
 //    m_send.m_ref = 0;
@@ -784,12 +947,12 @@ public:
       T_ROW_BUFFER_MAP = 0x100,
 
       /**
-       * Does any child need to know about when *my* batch is complete
+       * Does any child need to know when all its ancestors are complete
        */
       T_REPORT_BATCH_COMPLETE  = 0x200,
 
       /**
-       * Do I need to know when parent batch is completed
+       * Do *I need* to know when all ancestors has completed this batch
        */
       T_NEED_REPORT_BATCH_COMPLETED = 0x400,
 
@@ -813,10 +976,27 @@ public:
        */
       T_SCAN_REPEATABLE = 0x4000,
 
+      /**
+       * Exec of a previous REQ must complete before we can proceed.
+       * A ResumeEvent will later resume exec. of this operation
+       */
+      T_EXEC_SEQUENTIAL = 0x8000,
+
       // End marker...
       T_END = 0
     };
 
+    /**
+     * Describe whether a LQHKEY-REF and/or CONF whould trigger a 
+     * exec resume of another TreeNode having T_EXEC_SEQUENTIAL.
+     * (Used as a bitmask)
+     */
+    enum TreeNodeResumeEvents
+    {
+      TN_RESUME_REF   = 0x01,
+      TN_RESUME_CONF  = 0x02
+    };
+
     bool isLeaf() const { return (m_bits & T_LEAF) != 0;}
 
     // table or index this TreeNode operates on, and its schemaVersion
@@ -840,11 +1020,23 @@ public:
     /**
      * Rows buffered by this node
      */
-    union
-    {
-      RowMap m_row_map;
-      SLFifoRowList m_row_list;
-    };
+    RowCollection m_rows;
+
+    /**
+     * T_EXEC_SEQUENTIAL cause execution of child operations to
+     * be deferred.  These operations are queued in the 'struct DeferredParentOps'
+     * Currently only Lookup operation might be deferred.
+     * Could later be extended to also cover index scans.
+     */
+    DeferredParentOps m_deferred;
+
+    /**
+     * Set of TreeNodeResumeEvents, possibly or'ed.
+     * Specify whether a REF or CONF will cause a resume
+     * of the TreeNode referred by 'm_resumePtrI'.
+     */
+    Uint32 m_resumeEvents;
+    Uint32 m_resumePtrI;
 
     union
     {
@@ -892,7 +1084,7 @@ public:
       RT_SCAN                = 0x1  // unbounded result set, scan interface
       ,RT_ROW_BUFFERS        = 0x2  // Do any of the node use row-buffering
       ,RT_MULTI_SCAN         = 0x4  // Is there several scans in request
-      ,RT_VAR_ALLOC          = 0x8  // Is var-allocation used for row-buffer
+//    ,RT_VAR_ALLOC          = 0x8  // DEPRECATED
       ,RT_NEED_PREPARE       = 0x10 // Does any node need m_prepare hook
       ,RT_NEED_COMPLETE      = 0x20 // Does any node need m_complete hook
       ,RT_REPEAT_SCAN_RESULT = 0x40 // Repeat bushy scan result when required
@@ -929,6 +1121,8 @@ public:
     Uint32 m_cnt_active;       // No of "running" nodes
     TreeNodeBitMask
            m_active_nodes;     // Nodes which will return more data in NEXTREQ
+    TreeNodeBitMask
+           m_completed_nodes;  // Nodes wo/ any 'outstanding' signals
     Uint32 m_rows;             // Rows accumulated in current batch
     Uint32 m_outstanding;      // Outstanding signals, when 0, batch is done
     Uint16 m_lookup_node_data[MAX_NDB_NODES];
@@ -940,7 +1134,7 @@ public:
     Uint32 m_sum_rows;
     Uint32 m_sum_running;
     Uint32 m_sum_waiting;
-    Uint64 m_save_time;
+    NDB_TICKS m_save_time;
 #endif
 
     bool isScan() const { return (m_bits & RT_SCAN) != 0;}
@@ -1109,6 +1303,8 @@ private:
   void store_scan(Ptr<Request>);
   void handle_early_scanfrag_ref(Signal*, const ScanFragReq *, Uint32 err);
 
+  bool checkRequest(const Ptr<Request>);
+
   struct BuildKeyReq
   {
     Uint32 hashInfo[4]; // Used for hashing
@@ -1122,6 +1318,8 @@ private:
    */
   const OpInfo* getOpInfo(Uint32 op);
   Uint32 build(Build_context&,Ptr<Request>,SectionReader&,SectionReader&);
+  Uint32 initRowBuffers(Ptr<Request>);
+  void buildExecPlan(Ptr<Request>, Ptr<TreeNode> node, Ptr<TreeNode> next);
   void checkPrepareComplete(Signal*, Ptr<Request>, Uint32 cnt);
   void start(Signal*, Ptr<Request>);
   void checkBatchComplete(Signal*, Ptr<Request>, Uint32 cnt);
@@ -1130,15 +1328,16 @@ private:
   void sendConf(Signal*, Ptr<Request>, bool is_complete);
   void complete(Signal*, Ptr<Request>);
   void cleanup(Ptr<Request>);
+  void cleanupBatch(Ptr<Request>);
   void abort(Signal*, Ptr<Request>, Uint32 errCode);
   Uint32 nodeFail(Signal*, Ptr<Request>, NdbNodeBitmask mask);
 
   Uint32 createNode(Build_context&, Ptr<Request>, Ptr<TreeNode> &);
-  void reportBatchComplete(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void handleTreeNodeComplete(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void reportAncestorsComplete(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void releaseScanBuffers(Ptr<Request> requestPtr);
-  void releaseRequestBuffers(Ptr<Request> requestPtr, bool reset);
+  void releaseRequestBuffers(Ptr<Request> requestPtr);
   void releaseNodeRows(Ptr<Request> requestPtr, Ptr<TreeNode>);
-  void releaseRow(Ptr<Request>, RowRef ref);
   void registerActiveCursor(Ptr<Request>, Ptr<TreeNode>);
   void nodeFail_checkRequests(Signal*);
   void cleanup_common(Ptr<Request>, Ptr<TreeNode>);
@@ -1146,31 +1345,37 @@ private:
   /**
    * Row buffering
    */
-  Uint32 storeRow(Ptr<Request>, Ptr<TreeNode>, RowPtr &row);
+  Uint32 storeRow(RowCollection& collection, RowPtr &row);
+  void releaseRow(RowCollection& collection, RowRef ref);
   Uint32* stackAlloc(RowBuffer& dst, RowRef&, Uint32 len);
   Uint32* varAlloc(RowBuffer& dst, RowRef&, Uint32 len);
+  Uint32* rowAlloc(RowBuffer& dst, RowRef&, Uint32 len);
 
-  void add_to_list(SLFifoRowList & list, RowRef rowref);
-  Uint32 add_to_map(Ptr<Request> requestPtr, Ptr<TreeNode>, Uint32, RowRef);
-  Uint32 * get_row_ptr(const RowMap&, RowMapIterator pos);
-  void setupRowPtr(Ptr<TreeNode>, RowPtr& dst, RowRef, const Uint32 * src);
+  void add_to_list(SLFifoRowList & list, RowRef);
+  Uint32 add_to_map(RowMap& map, Uint32, RowRef);
 
-  // NOTE: ref contains info about it being stack/var
-  // so adding an inline would be nice...but that remove possibility
-  // to add jam()'s
-  Uint32 * get_row_ptr_stack(RowRef pos);
-  Uint32 * get_row_ptr_var(RowRef pos);
+  void setupRowPtr(const RowCollection& collection,
+                   RowPtr& dst, RowRef, const Uint32 * src);
+  Uint32 * get_row_ptr(RowRef pos);
 
   /**
    * SLFifoRowListIterator
    */
-  bool first(Ptr<Request>, Ptr<TreeNode>, SLFifoRowListIterator&);
+  bool first(const SLFifoRowList& list, SLFifoRowListIterator&);
   bool next(SLFifoRowListIterator&);
-  bool next(Ptr<Request>, Ptr<TreeNode>, SLFifoRowListIterator&, SLFifoRowListIteratorPtr);
 
-  bool first(Ptr<Request>, Ptr<TreeNode>, RowMapIterator&);
+  /**
+   * RowMapIterator
+   */
+  bool first(const RowMap& map, RowMapIterator&);
   bool next(RowMapIterator&);
-  bool next(Ptr<Request>,Ptr<TreeNode>, RowMapIterator&, RowMapIteratorPtr);
+
+  /**
+   * RowIterator:
+   * Abstraction which may iterate either a RowList or Map
+   */
+  bool first(const RowCollection&, RowIterator&);
+  bool next(RowIterator&);
 
   /**
    * Misc
@@ -1220,13 +1425,19 @@ private:
                  DABuffer & tree, Uint32 treeBits,
                  DABuffer & param, Uint32 paramBits);
 
-  Uint32 createEmptySection(Uint32 & ptrI);
-
   Uint32 getResultRef(Ptr<Request> requestPtr);
 
   Uint32 checkTableError(Ptr<TreeNode> treeNodePtr) const;
   Uint32 getNodes(Signal*, BuildKeyReq&, Uint32 tableId);
 
+  void common_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
+			     const RowPtr&);
+
+  void dumpScanFragHandle(const Ptr<ScanFragHandle> fragPtr) const;
+  void dumpNodeCommon(const Ptr<TreeNode>) const;
+  void dumpRequest(const char* reason,
+                   const Ptr<Request> requestPtr);
+
   /**
    * Lookup
    */
@@ -1234,16 +1445,21 @@ private:
   Uint32 lookup_build(Build_context&,Ptr<Request>,
 		      const QueryNode*, const QueryNodeParameters*);
   void lookup_start(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void lookup_resume(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_send(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execTRANSID_AI(Signal*, Ptr<Request>, Ptr<TreeNode>,
 			     const RowPtr&);
   void lookup_execLQHKEYREF(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void lookup_execLQHKEYCONF(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void lookup_stop_branch(Signal*, Ptr<Request>, Ptr<TreeNode>, Uint32 err);
   void lookup_parent_row(Signal*, Ptr<Request>, Ptr<TreeNode>, const RowPtr &);
-  void lookup_parent_batch_complete(Signal*, Ptr<Request>, Ptr<TreeNode>);
+  void lookup_row(Signal*, Ptr<Request>, Ptr<TreeNode>, const RowPtr &);
   void lookup_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
   Uint32 lookup_execNODE_FAILREP(Signal*signal, Ptr<Request>, Ptr<TreeNode>,
                                NdbNodeBitmask);
+
+  void lookup_sendLeafCONF(Signal*, Ptr<Request>, Ptr<TreeNode>,
+                           Uint32 node);
   void lookup_cleanup(Ptr<Request>, Ptr<TreeNode>);
 
   Uint32 handle_special_hash(Uint32 tableId, Uint32 dstHash[4],
@@ -1253,6 +1469,10 @@ private:
 
   Uint32 computeHash(Signal*, BuildKeyReq&, Uint32 table, Uint32 keyInfoPtrI);
   Uint32 computePartitionHash(Signal*, BuildKeyReq&, Uint32 table, Uint32 keyInfoPtrI);
+  bool lookup_checkNode(const Ptr<Request> requestPtr, 
+                        const Ptr<TreeNode> treeNodePtr);
+  void lookup_dumpNode(const Ptr<Request> requestPtr,
+                       const Ptr<TreeNode> treeNodePtr);
 
   /**
    * ScanFrag
@@ -1269,6 +1489,10 @@ private:
   void scanFrag_execSCAN_NEXTREQ(Signal*, Ptr<Request>,Ptr<TreeNode>);
   void scanFrag_abort(Signal*, Ptr<Request>, Ptr<TreeNode>);
   void scanFrag_cleanup(Ptr<Request>, Ptr<TreeNode>);
+  bool scanFrag_checkNode(const Ptr<Request> requestPtr, 
+                          const Ptr<TreeNode> treeNodePtr);
+  void scanFrag_dumpNode(const Ptr<Request> requestPtr,
+                         const Ptr<TreeNode> treeNodePtr);
 
   /**
    * ScanIndex
@@ -1312,6 +1536,12 @@ private:
                                       Ptr<Request> requestPtr,
                                       Ptr<TreeNode> treeNodePtr);
 
+  bool scanIndex_checkNode(const Ptr<Request> requestPtr,
+                           const Ptr<TreeNode> treeNodePtr);
+
+  void scanIndex_dumpNode(const Ptr<Request> requestPtr,
+                          const Ptr<TreeNode> treeNodePtr);
+
   /**
    * Page manager
    */
@@ -1336,4 +1566,7 @@ private:
   Uint32 m_buffer1[16*1024]; // 64k
 };
 
+
+#undef JAM_FILE_ID
+
 #endif
