@@ -1,19 +1,26 @@
 /*
-   Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #define DBTUP_C
 #define DBTUP_COMMIT_CPP
@@ -22,10 +29,12 @@
 #include <ndb_limits.h>
 #include <pc.hpp>
 #include <signaldata/TupCommit.hpp>
+#include <EventLogger.hpp>
 #include "../dblqh/Dblqh.hpp"
 
 #define JAM_FILE_ID 416
 
+extern EventLogger *g_eventLogger;
 
 void Dbtup::execTUP_DEALLOCREQ(Signal* signal)
 {
@@ -633,6 +642,8 @@ int Dbtup::retrieve_log_page(Signal *signal,
     signal->theData[0] = 1;
     return res;
   case -1:
+    g_eventLogger->warning("Out of space in RG_DISK_OPERATIONS resource,"
+                           " increase config parameter GlobalSharedMemory");
     ndbrequire("NOT YET IMPLEMENTED" == 0);
     break;
   default:
