@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -35,9 +42,12 @@
 #include <signaldata/NodeStateSignalData.hpp>
 
 #include <signaldata/DropTab.hpp>
-#include <SLList.hpp>
+#include <IntrusiveList.hpp>
 
 #include <EventLogger.hpp>
+
+#define JAM_FILE_ID 420
+
 extern EventLogger * g_eventLogger;
 
 #define DEBUG(x) { ndbout << "TUP::" << x << endl; }
@@ -45,6 +55,7 @@ extern EventLogger * g_eventLogger;
 void Dbtup::initData() 
 {
   TablerecPtr tablePtr;
+  (void)tablePtr; // hide unused warning
   cnoOfFragrec = NDB_ARRAY_SIZE(tablePtr.p->fragrec);
   cnoOfFragoprec = NDB_ARRAY_SIZE(tablePtr.p->fragrec);
   cnoOfAlterTabOps = NDB_ARRAY_SIZE(tablePtr.p->fragrec);
@@ -66,6 +77,7 @@ void Dbtup::initData()
 Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
   : SimulatedBlock(DBTUP, ctx, instanceNumber),
     c_lqh(0),
+    c_backup(0),
     c_tsman(0),
     c_lgman(0),
     c_pgman(0),
@@ -190,7 +202,7 @@ Dbtup::Dbtup(Block_context& ctx, Uint32 instanceNumber)
   { // 7
     CallbackEntry& ce = m_callbackEntry[DISK_PAGE_LOG_BUFFER_CALLBACK];
     ce.m_function = safe_cast(&Dbtup::disk_page_log_buffer_callback);
-    ce.m_flags = 0;
+    ce.m_flags = CALLBACK_ACK;
   }
   {
     CallbackTable& ct = m_callbackTable;
@@ -361,6 +373,7 @@ void Dbtup::execSTTOR(Signal* signal)
   case ZSTARTPHASE1:
     jam();
     ndbrequire((c_lqh= (Dblqh*)globalData.getBlock(DBLQH, instance())) != 0);
+    ndbrequire((c_backup= (Backup*)globalData.getBlock(BACKUP, instance())) != 0);
     ndbrequire((c_tsman= (Tsman*)globalData.getBlock(TSMAN)) != 0);
     ndbrequire((c_lgman= (Lgman*)globalData.getBlock(LGMAN)) != 0);
     ndbrequire((c_pgman= (Pgman*)globalData.getBlock(PGMAN, instance())) != 0);
@@ -806,21 +819,21 @@ Dbtup::initTab(Tablerec* const regTabPtr)
 
   // Clear trigger data
   if (!regTabPtr->afterInsertTriggers.isEmpty())
-    regTabPtr->afterInsertTriggers.release();
+    while (regTabPtr->afterInsertTriggers.releaseFirst());
   if (!regTabPtr->afterDeleteTriggers.isEmpty())
-    regTabPtr->afterDeleteTriggers.release();
+    while (regTabPtr->afterDeleteTriggers.releaseFirst());
   if (!regTabPtr->afterUpdateTriggers.isEmpty())
-    regTabPtr->afterUpdateTriggers.release();
+    while (regTabPtr->afterUpdateTriggers.releaseFirst());
   if (!regTabPtr->subscriptionInsertTriggers.isEmpty())
-    regTabPtr->subscriptionInsertTriggers.release();
+    while (regTabPtr->subscriptionInsertTriggers.releaseFirst());
   if (!regTabPtr->subscriptionDeleteTriggers.isEmpty())
-    regTabPtr->subscriptionDeleteTriggers.release();
+    while (regTabPtr->subscriptionDeleteTriggers.releaseFirst());
   if (!regTabPtr->subscriptionUpdateTriggers.isEmpty())
-    regTabPtr->subscriptionUpdateTriggers.release();
+    while (regTabPtr->subscriptionUpdateTriggers.releaseFirst());
   if (!regTabPtr->constraintUpdateTriggers.isEmpty())
-    regTabPtr->constraintUpdateTriggers.release();
+    while (regTabPtr->constraintUpdateTriggers.releaseFirst());
   if (!regTabPtr->tuxCustomTriggers.isEmpty())
-    regTabPtr->tuxCustomTriggers.release();
+    while (regTabPtr->tuxCustomTriggers.releaseFirst());
 }//Dbtup::initTab()
 
 void Dbtup::initializeTabDescr() 
