@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -19,8 +26,12 @@
 
 #include "Ndbfs.hpp"
 #include "AsyncFile.hpp"
+
+#ifdef NDB_WIN
+#include "Win32AsyncFile.hpp"
+#else
 #include "PosixAsyncFile.hpp"
-#include "Filename.hpp"
+#endif
 
 #include <signaldata/FsOpenReq.hpp>
 #include <signaldata/FsCloseReq.hpp>
@@ -32,12 +43,21 @@
 #include <signaldata/NdbfsContinueB.hpp>
 #include <signaldata/DumpStateOrd.hpp>
 #include <signaldata/AllocMem.hpp>
+#include <signaldata/BuildIndxImpl.hpp>
 
 #include <RefConvert.hpp>
-#include <NdbSleep.h>
+#include <portlib/NdbDir.hpp>
 #include <NdbOut.hpp>
 #include <Configuration.hpp>
 
+#include <EventLogger.hpp>
+
+#define JAM_FILE_ID 393
+
+extern EventLogger * g_eventLogger;
+
+NdbMutex g_active_bound_threads_mutex;
+
 inline
 int pageSize( const NewVARIABLE* baseAddrRef )
 {
@@ -50,16 +70,20 @@ int pageSize( const NewVARIABLE* baseAddrRef )
    return (1 << log_psize);
 }
 
-
 Ndbfs::Ndbfs(Block_context& ctx) :
   SimulatedBlock(NDBFS, ctx),
-  theLastId(0),
   scanningInProgress(false),
+  theLastId(0),
   theRequestPool(0),
-  m_maxOpenedFiles(0)
+  m_maxOpenedFiles(0),
+  m_bound_threads_cnt(0),
+  m_unbounds_threads_cnt(0),
+  m_active_bound_threads_cnt(0)
 {
   BLOCK_CONSTRUCTOR(Ndbfs);
 
+  NdbMutex_Init(&g_active_bound_threads_mutex);
+
   // Set received signals
   addRecSignal(GSN_READ_CONFIG_REQ, &Ndbfs::execREAD_CONFIG_REQ);
   addRecSignal(GSN_DUMP_STATE_ORD,  &Ndbfs::execDUMP_STATE_ORD);
@@ -74,7 +98,7 @@ Ndbfs::Ndbfs(Block_context& ctx) :
   addRecSignal(GSN_FSREMOVEREQ, &Ndbfs::execFSREMOVEREQ);
   addRecSignal(GSN_ALLOC_MEM_REQ, &Ndbfs::execALLOC_MEM_REQ);
   addRecSignal(GSN_SEND_PACKED, &Ndbfs::execSEND_PACKED, true);
-  addRecSignal(GSN_BUILDINDXREQ, &Ndbfs::execBUILDINDXREQ);
+  addRecSignal(GSN_BUILD_INDX_IMPL_REQ, &Ndbfs::execBUILD_INDX_IMPL_REQ);
    // Set send signals
   addRecSignal(GSN_FSSUSPENDORD, &Ndbfs::execFSSUSPENDORD);
 
@@ -83,16 +107,48 @@ Ndbfs::Ndbfs(Block_context& ctx) :
 
 Ndbfs::~Ndbfs()
 {
-  // Delete all files
-  // AsyncFile destuctor will take care of deleting
-  // the thread it has created
+  /**
+   * Stop all unbound threads
+   */
+
+  /**
+   * Post enought Request::end to saturate all unbound threads
+   */
+  Request request;
+  request.action = Request::end;
+  for (unsigned i = 0; i < theThreads.size(); i++)
+  {
+    theToBoundThreads.writeChannel(&request);
+    theToUnboundThreads.writeChannel(&request);
+  }
+
+  for (unsigned i = 0; i < theThreads.size(); i++)
+  {
+    AsyncIoThread * thr = theThreads[i];
+    thr->shutdown();
+  }
+
+  /**
+   * delete all threads
+   */
+  for (unsigned i = 0; i < theThreads.size(); i++)
+  {
+    AsyncIoThread * thr = theThreads[i];
+    delete thr;
+    theThreads[i] = 0;
+  }
+  theThreads.clear();
+
+  /**
+   * Delete all files
+   */
   for (unsigned i = 0; i < theFiles.size(); i++){
     AsyncFile* file = theFiles[i];
-    file->shutdown();
     delete file;
     theFiles[i] = NULL;
   }//for
   theFiles.clear();
+
   if (theRequestPool)
     delete theRequestPool;
 }
@@ -101,11 +157,9 @@ static
 bool
 do_mkdir(const char * path)
 {
-#ifdef NDB_WIN32
-  return CreateDirectory(path, 0);
-#else
-  return ::mkdir(path, S_IRUSR | S_IWUSR | S_IXUSR | S_IXGRP | S_IRGRP) == 0;
-#endif
+  return NdbDir::create(path,
+                        NdbDir::u_rwx() | NdbDir::g_r() | NdbDir::g_x(),
+                        true /* ignore_existing */);
 }
 
 static
@@ -230,13 +284,56 @@ Ndbfs::execREAD_CONFIG_REQ(Signal* signal)
   m_maxFiles = 0;
   ndb_mgm_get_int_parameter(p, CFG_DB_MAX_OPEN_FILES, &m_maxFiles);
   Uint32 noIdleFiles = 27;
+
   ndb_mgm_get_int_parameter(p, CFG_DB_INITIAL_OPEN_FILES, &noIdleFiles);
+
+  {
+    /**
+     * each logpart keeps up to 3 logfiles open at any given time...
+     *   (bound)
+     * make sure noIdleFiles is atleast 4 times #logparts
+     */
+    Uint32 logParts = NDB_DEFAULT_LOG_PARTS;
+    ndb_mgm_get_int_parameter(p, CFG_DB_NO_REDOLOG_PARTS, &logParts);
+    Uint32 logfiles = 4 * logParts;
+    if (noIdleFiles < logfiles)
+    {
+      noIdleFiles = logfiles;
+    }
+  }
+
   // Make sure at least "noIdleFiles" files can be created
   if (noIdleFiles > m_maxFiles && m_maxFiles != 0)
     m_maxFiles = noIdleFiles;
+
   // Create idle AsyncFiles
-  for (Uint32 i = 0; i < noIdleFiles; i++){
+  for (Uint32 i = 0; i < noIdleFiles; i++)
+  {
     theIdleFiles.push_back(createAsyncFile());
+    AsyncIoThread * thr = createIoThread(/* bound */ true);
+    if (thr)
+    {
+      theThreads.push_back(thr);
+    }
+  }
+
+  Uint32 threadpool = 2;
+  ndb_mgm_get_int_parameter(p, CFG_DB_THREAD_POOL, &threadpool);
+
+  // Create IoThreads
+  for (Uint32 i = 0; i < threadpool; i++)
+  {
+    AsyncIoThread * thr = createIoThread(/* bound */ false);
+    if (thr)
+    {
+      jam();
+      theThreads.push_back(thr);
+    }
+    else
+    {
+      jam();
+      break;
+    }
   }
 
   setup_wakeup();
@@ -270,6 +367,46 @@ Ndbfs::execSTTOR(Signal* signal)
   if(signal->theData[1] == 0){ // StartPhase 0
     jam();
     
+    if (ERROR_INSERTED(2000) || ERROR_INSERTED(2001))
+    {
+      // Save(2000) or restore(2001) FileSystemPath/ndb_XX_fs/
+      BaseString& fs_path = m_base_path[FsOpenReq::BP_FS];
+      unsigned i = fs_path.length() - strlen(DIR_SEPARATOR);
+      BaseString saved_path(fs_path.c_str(), i);
+      const char * ending_separator = fs_path.c_str() + i;
+      ndbrequire(strcmp(ending_separator, DIR_SEPARATOR)==0);
+      saved_path.append(".saved");
+      saved_path.append(ending_separator);
+      BaseString& from_dir = (ERROR_INSERTED(2000) ? fs_path : saved_path);
+      BaseString& to_dir = (ERROR_INSERTED(2000) ? saved_path : fs_path);
+
+      const bool only_contents = true;
+      if (NdbDir::remove_recursive(to_dir.c_str(), !only_contents))
+      {
+        g_eventLogger->info("Cleaned destination file system at %s", to_dir.c_str());
+      }
+      else
+      {
+        g_eventLogger->warning("Failed cleaning file system at %s", to_dir.c_str());
+      }
+      if (access(to_dir.c_str(), F_OK) == 0 || errno != ENOENT)
+      {
+        g_eventLogger->error("Destination file system at %s should not be there (errno %d)!",
+                             to_dir.c_str(),
+                             errno);
+        ndbrequire(!"Destination file system already there during file system saving or restoring");
+      }
+      if (rename(from_dir.c_str(), to_dir.c_str()) == -1)
+      {
+        g_eventLogger->error("Failed renaming %s file system to %s while %s (errno %d)",
+          from_dir.c_str(),
+          to_dir.c_str(),
+          (ERROR_INSERTED(2000) ? "saving" : "restoring"),
+          errno);
+        ndbrequire(!"Failed renaming file system while saving ot restoring");
+      }
+      SET_ERROR_INSERT_VALUE2(ERROR_INSERT_EXTRA, 0);
+    }
     do_mkdir(m_base_path[FsOpenReq::BP_FS].c_str());
     
     // close all open files
@@ -282,11 +419,23 @@ Ndbfs::execSTTOR(Signal* signal)
   ndbrequire(0);
 }
 
-int 
+int
 Ndbfs::forward( AsyncFile * file, Request* request)
 {
   jam();
-  file->execute(request);
+  AsyncIoThread* thr = file->getThread();
+  if (thr) // bound
+  {
+    thr->dispatch(request);
+  }
+  else if (request->m_do_bind)
+  {
+    theToBoundThreads.writeChannel(request);
+  }
+  else
+  {
+    theToUnboundThreads.writeChannel(request);
+  }
   return 1;
 }
 
@@ -296,12 +445,25 @@ Ndbfs::execFSOPENREQ(Signal* signal)
   jamEntry();
   const FsOpenReq * const fsOpenReq = (FsOpenReq *)&signal->theData[0];
   const BlockReference userRef = fsOpenReq->userReference;
-  AsyncFile* file = getIdleFile();
+
+  bool bound = (fsOpenReq->fileFlags & FsOpenReq::OM_THREAD_POOL) == 0;
+  AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
 
   Uint32 userPointer = fsOpenReq->userPointer;
   
-  if(fsOpenReq->fileFlags & FsOpenReq::OM_INIT)
+  
+  SectionHandle handle(this, signal);
+  SegmentedSectionPtr ptr; ptr.setNull();
+  if (handle.m_cnt)
+  {
+    jam();
+    handle.getSection(ptr, FsOpenReq::FILENAME);
+  }
+  file->theFileName.set(this, userRef, fsOpenReq->fileNumber, false, ptr);
+  releaseSections(handle);
+  
+  if (fsOpenReq->fileFlags & FsOpenReq::OM_INIT)
   {
     jam();
     Uint32 cnt = 16; // 512k
@@ -320,9 +482,30 @@ Ndbfs::execFSOPENREQ(Signal* signal)
       return;
     }
     m_shared_page_pool.getPtr(page_ptr);
-    file->m_page_ptr = page_ptr;
-    file->m_page_cnt = cnt;
+    file->set_buffer(RT_DBTUP_PAGE, page_ptr, cnt);
   } 
+  else if (fsOpenReq->fileFlags & FsOpenReq::OM_WRITE_BUFFER)
+  {
+    jam();
+    Uint32 cnt = NDB_FILE_BUFFER_SIZE / GLOBAL_PAGE_SIZE; // 256k
+    Ptr<GlobalPage> page_ptr;
+    m_ctx.m_mm.alloc_pages(RT_FILE_BUFFER, &page_ptr.i, &cnt, 1);
+    if (cnt == 0)
+    {
+      jam();
+      file->m_page_ptr.setNull();
+      file->m_page_cnt = 0;
+
+      FsRef * const fsRef = (FsRef *)&signal->theData[0];
+      fsRef->userPointer  = userPointer;
+      fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrOutOfMemory);
+      fsRef->osErrorCode  = ~0; // Indicate local error
+      sendSignal(userRef, GSN_FSOPENREF, signal, 3, JBB);
+      return;
+    }
+    m_shared_page_pool.getPtr(page_ptr);
+    file->set_buffer(RT_FILE_BUFFER, page_ptr, cnt);
+  }
   else
   {
     ndbassert(file->m_page_ptr.isNull());
@@ -330,20 +513,8 @@ Ndbfs::execFSOPENREQ(Signal* signal)
     file->m_page_cnt = 0;
   }
   
-  SectionHandle handle(this, signal);
-  SegmentedSectionPtr ptr; ptr.setNull();
-  if (handle.m_cnt)
-  {
-    jam();
-    handle.getSection(ptr, FsOpenReq::FILENAME);
-  }
-
-  file->theFileName.set(this, userRef, fsOpenReq->fileNumber, false, ptr);
-  releaseSections(handle);
-
-  file->reportTo(&theFromThreads);
   if (getenv("NDB_TRACE_OPEN"))
-    ndbout_c("open(%s)", file->theFileName.c_str());
+    ndbout_c("open(%s) bound: %u", file->theFileName.c_str(), bound);
   
   Request* request = theRequestPool->get();
   request->action = Request::open;
@@ -357,7 +528,8 @@ Ndbfs::execFSOPENREQ(Signal* signal)
   request->par.open.file_size <<= 32;
   request->par.open.file_size |= fsOpenReq->file_size_lo;
   request->par.open.auto_sync_size = fsOpenReq->auto_sync_size;
-  
+  request->m_do_bind = bound;
+
   ndbrequire(forward(file, request));
 }
 
@@ -367,7 +539,8 @@ Ndbfs::execFSREMOVEREQ(Signal* signal)
   jamEntry();
   const FsRemoveReq * const req = (FsRemoveReq *)signal->getDataPtr();
   const BlockReference userRef = req->userReference;
-  AsyncFile* file = getIdleFile();
+  bool bound = true;
+  AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
 
   SectionHandle handle(this, signal);
@@ -384,8 +557,6 @@ Ndbfs::execFSREMOVEREQ(Signal* signal)
   Uint32 version = FsOpenReq::getVersion(req->fileNumber);
   Uint32 bp = FsOpenReq::v5_getLcpNo(req->fileNumber);
 
-  file->reportTo(&theFromThreads);
-
   Request* request = theRequestPool->get();
   request->action = Request::rmrf;
   request->par.rmrf.directory = req->directory;
@@ -394,7 +565,8 @@ Ndbfs::execFSREMOVEREQ(Signal* signal)
   request->set(userRef, req->userPointer, newId() );
   request->file = file;
   request->theTrace = signal->getTrace();
-  
+  request->m_do_bind = bound;
+
   if (version == 6)
   {
     ndbrequire(bp < NDB_ARRAY_SIZE(m_base_path));
@@ -433,9 +605,17 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
     fsRef->setErrorCode(fsRef->errorCode, FsRef::fsErrFileDoesNotExist);
     fsRef->osErrorCode  = ~0; // Indicate local error
     sendSignal(userRef, GSN_FSCLOSEREF, signal, 3, JBB);
+
+    g_eventLogger->warning("Trying to close unknown file!! %u", userPointer);
+    g_eventLogger->warning("Dumping files");
+    signal->theData[0] = 405;
+    execDUMP_STATE_ORD(signal);
     return;
   }
 
+  if (getenv("NDB_TRACE_OPEN"))
+    ndbout_c("close(%s)", openFile->theFileName.c_str());
+
   Request *request = theRequestPool->get();
   if( fsCloseReq->getRemoveFileFlag(fsCloseReq->fileFlag) == true ) {
      jam();
@@ -448,6 +628,7 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
   request->file = openFile;
   request->error = 0;
   request->theTrace = signal->getTrace();
+  request->m_do_bind = false;
 
   ndbrequire(forward(openFile, request));
 }
@@ -455,15 +636,29 @@ Ndbfs::execFSCLOSEREQ(Signal * signal)
 void 
 Ndbfs::readWriteRequest(int action, Signal * signal)
 {
-  const FsReadWriteReq * const fsRWReq = (FsReadWriteReq *)&signal->theData[0];
+  Uint32 theData[25 + 2 * NDB_FS_RW_PAGES];
+  memcpy(theData, signal->theData, 4 * signal->getLength());
+  SectionHandle handle(this, signal);
+  if (handle.m_cnt > 0)
+  {
+    SegmentedSectionPtr secPtr;
+    ndbrequire(handle.getSection(secPtr, 0));
+    ndbrequire(signal->getLength() + secPtr.sz < NDB_ARRAY_SIZE(theData));
+    copy(theData + signal->getLength(), secPtr);
+    releaseSections(handle);
+  }
+
+  const FsReadWriteReq * const fsRWReq = (FsReadWriteReq *)theData;
   Uint16 filePointer =  (Uint16)fsRWReq->filePointer;
   const UintR userPointer = fsRWReq->userPointer; 
   const BlockReference userRef = fsRWReq->userReference;
-  const BlockNumber blockNumber = refToBlock(userRef);
+  const BlockNumber blockNumber = refToMain(userRef);
+  const Uint32 instanceNumber = refToInstance(userRef);
 
   AsyncFile* openFile = theOpenFiles.find(filePointer);
 
-  const NewVARIABLE *myBaseAddrRef = &getBat(blockNumber)[fsRWReq->varIndex];
+  const NewVARIABLE *myBaseAddrRef =
+    &getBat(blockNumber, instanceNumber)[fsRWReq->varIndex];
   UintPtr tPageSize;
   UintPtr tClusterSize;
   UintPtr tNRR;
@@ -477,6 +672,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   request->file = openFile;
   request->action = (Request::Action) action;
   request->theTrace = signal->getTrace();
+  request->m_do_bind = false;
 
   Uint32 format = fsRWReq->getFormatFlag(fsRWReq->operationFlag);
 
@@ -489,7 +685,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
   if(format != FsReadWriteReq::fsFormatGlobalPage &&
      format != FsReadWriteReq::fsFormatSharedPage)
   {     
-    if (fsRWReq->varIndex >= getBatSize(blockNumber)) {
+    if (fsRWReq->varIndex >= getBatSize(blockNumber, instanceNumber)) {
       jam();// Ensure that a valid variable is used    
       errorCode = FsRef::fsErrInvalidParameters;
       goto error;
@@ -525,7 +721,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	}//if
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = fileOffset * tPageSize;
+	request->par.readWrite.pages[i].offset = (off_t)(fileOffset*tPageSize);
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -541,7 +737,7 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
       const UintPtr varIndex = fsRWReq->data.arrayOfPages.varIndex;
       const UintPtr fileOffset = fsRWReq->data.arrayOfPages.fileOffset;
       
-      request->par.readWrite.pages[0].offset = fileOffset * tPageSize;
+      request->par.readWrite.pages[0].offset = (off_t)(fileOffset * tPageSize);
       request->par.readWrite.pages[0].size = tPageSize * fsRWReq->numberOfPages;
       request->par.readWrite.numberOfPages = 1;
       request->par.readWrite.pages[0].buf = &tWA[varIndex * tPageSize];
@@ -565,7 +761,8 @@ Ndbfs::readWriteRequest(int action, Signal * signal)
 	}//if
 	request->par.readWrite.pages[i].buf = &tWA[varIndex * tClusterSize];
 	request->par.readWrite.pages[i].size = tPageSize;
-	request->par.readWrite.pages[i].offset = tPageOffset + (i*tPageSize);
+	request->par.readWrite.pages[i].offset = (off_t)
+          (tPageOffset + (i*tPageSize));
       }//for
       request->par.readWrite.numberOfPages = fsRWReq->numberOfPages;
       break;
@@ -696,7 +893,8 @@ Ndbfs::execFSSYNCREQ(Signal * signal)
   request->set(userRef, userPointer, filePointer);
   request->file = openFile;
   request->theTrace = signal->getTrace();
-  
+  request->m_do_bind = false;
+
   ndbrequire(forward(openFile,request));
 }
 
@@ -724,6 +922,7 @@ Ndbfs::execFSSUSPENDORD(Signal * signal)
   request->file = openFile;
   request->theTrace = signal->getTrace();
   request->par.suspend.milliseconds = millis;
+  request->m_do_bind = false;
 
   ndbrequire(forward(openFile,request));
 }
@@ -735,12 +934,14 @@ Ndbfs::execFSAPPENDREQ(Signal * signal)
   const Uint16 filePointer =  (Uint16)fsReq->filePointer;
   const UintR userPointer = fsReq->userPointer; 
   const BlockReference userRef = fsReq->userReference;
-  const BlockNumber blockNumber = refToBlock(userRef);
+  const BlockNumber blockNumber = refToMain(userRef);
+  const Uint32 instanceNumber = refToInstance(userRef);
 
   FsRef::NdbfsErrorCodeType errorCode;
 
   AsyncFile* openFile = theOpenFiles.find(filePointer);
-  const NewVARIABLE *myBaseAddrRef = &getBat(blockNumber)[fsReq->varIndex];
+  const NewVARIABLE *myBaseAddrRef =
+    &getBat(blockNumber, instanceNumber)[fsReq->varIndex];
 
   const Uint32* tWA   = (const Uint32*)myBaseAddrRef->WA;
   const Uint32  tSz   = myBaseAddrRef->nrr;
@@ -761,7 +962,7 @@ Ndbfs::execFSAPPENDREQ(Signal * signal)
     goto error;
   }
   
-  if (fsReq->varIndex >= getBatSize(blockNumber)) {
+  if (fsReq->varIndex >= getBatSize(blockNumber, instanceNumber)) {
     jam();// Ensure that a valid variable is used    
     errorCode = FsRef::fsErrInvalidParameters;
     goto error;
@@ -785,6 +986,7 @@ Ndbfs::execFSAPPENDREQ(Signal * signal)
     request->action = Request::append;
   else
     request->action = Request::append_synch;
+  request->m_do_bind = false;
   ndbrequire(forward(openFile, request));
   return;
   
@@ -808,9 +1010,9 @@ Ndbfs::execALLOC_MEM_REQ(Signal* signal)
 
   AllocMemReq* req = (AllocMemReq*)signal->getDataPtr();
 
-  AsyncFile* file = getIdleFile();
+  bool bound = true;
+  AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
-  file->reportTo(&theFromThreads);
 
   Request *request = theRequestPool->get();
 
@@ -821,21 +1023,21 @@ Ndbfs::execALLOC_MEM_REQ(Signal* signal)
 
   request->par.alloc.ctx = &m_ctx;
   request->par.alloc.requestInfo = req->requestInfo;
+  request->par.alloc.bytes = (Uint64(req->bytes_hi) << 32) + req->bytes_lo;
   request->action = Request::allocmem;
+  request->m_do_bind = bound;
   ndbrequire(forward(file, request));
 }
 
-#include <signaldata/BuildIndx.hpp>
-
 void
-Ndbfs::execBUILDINDXREQ(Signal* signal)
+Ndbfs::execBUILD_INDX_IMPL_REQ(Signal* signal)
 {
   jamEntry();
   mt_BuildIndxReq * req = (mt_BuildIndxReq*)signal->getDataPtr();
 
-  AsyncFile* file = getIdleFile();
+  bool bound = true;
+  AsyncFile* file = getIdleFile(bound);
   ndbrequire(file != NULL);
-  file->reportTo(&theFromThreads);
 
   Request *request = theRequestPool->get();
   request->error = 0;
@@ -859,11 +1061,11 @@ Ndbfs::execBUILDINDXREQ(Signal* signal)
   ndbrequire(cnt == save);
 
   m_shared_page_pool.getPtr(page_ptr);
-  file->m_page_ptr = page_ptr;
-  file->m_page_cnt = cnt;
+  file->set_buffer(RT_DBTUP_PAGE, page_ptr, cnt);
 
   memcpy(&request->par.build.m_req, req, sizeof(* req));
   request->action = Request::buildindx;
+  request->m_do_bind = bound;
   ndbrequire(forward(file, request));
 }
 
@@ -892,10 +1094,11 @@ Ndbfs::newId()
 }
 
 AsyncFile*
-Ndbfs::createAsyncFile(){
-
+Ndbfs::createAsyncFile()
+{
   // Check limit of open files
-  if (m_maxFiles !=0 && theFiles.size() ==  m_maxFiles) {
+  if (m_maxFiles !=0 && theFiles.size() ==  m_maxFiles)
+  {
     // Print info about all open files
     for (unsigned i = 0; i < theFiles.size(); i++){
       AsyncFile* file = theFiles[i];
@@ -904,32 +1107,99 @@ Ndbfs::createAsyncFile(){
     ERROR_SET(fatal, NDBD_EXIT_AFS_MAXOPEN,""," Ndbfs::createAsyncFile");
   }
 
+#ifdef NDB_WIN
+  AsyncFile* file = new Win32AsyncFile(* this);
+#else
   AsyncFile* file = new PosixAsyncFile(* this);
-  file->doStart();
-
-  // Put the file in list of all files
-  theFiles.push_back(file);
-
-#ifdef VM_TRACE
-  infoEvent("NDBFS: Created new file thread %d", theFiles.size());
 #endif
-  
+
+  if (file->init())
+  {
+    ERROR_SET(fatal, NDBD_EXIT_AFS_MAXOPEN,""," Ndbfs::createAsyncFile");
+  }
+
+  theFiles.push_back(file);
   return file;
 }
 
+void
+Ndbfs::pushIdleFile(AsyncFile* file)
+{
+  assert(file->getThread() == 0);
+  theIdleFiles.push_back(file);
+}
+
+AsyncIoThread*
+Ndbfs::createIoThread(bool bound)
+{
+  AsyncIoThread* thr = new AsyncIoThread(*this, bound);
+  if (thr)
+  {
+#ifdef VM_TRACE
+    ndbout_c("NDBFS: Created new file thread %d", theThreads.size());
+#endif
+
+    struct NdbThread* thrptr = thr->doStart();
+    globalEmulatorData.theConfiguration->addThread(thrptr, NdbfsThread);
+    thr->set_real_time(
+      globalEmulatorData.theConfiguration->get_io_real_time());
+
+    if (bound)
+      m_bound_threads_cnt++;
+    else
+      m_unbounds_threads_cnt++;
+  }
+
+  return thr;
+}
+
 AsyncFile*
-Ndbfs::getIdleFile(){
-  AsyncFile* file;
-  if (theIdleFiles.size() > 0){
-    file = theIdleFiles[0];
-    theIdleFiles.erase(0);
-  } else {
+Ndbfs::getIdleFile(bool bound)
+{
+  AsyncFile* file = 0;
+  Uint32 sz = theIdleFiles.size();
+  if (sz)
+  {
+    file = theIdleFiles[sz - 1];
+    theIdleFiles.erase(sz - 1);
+  }
+  else
+  {
     file = createAsyncFile();
-  } 
+  }
+
+  if (bound)
+  {
+    /**
+     * Check if we should create thread
+     */
+    if (m_active_bound_threads_cnt == m_bound_threads_cnt)
+    {
+      AsyncIoThread * thr = createIoThread(true);
+      if (thr)
+      {
+        theThreads.push_back(thr);
+      }
+    }
+  }
   return file;
 }
 
-
+void
+Ndbfs::cnt_active_bound(int val)
+{
+  Guard g(&g_active_bound_threads_mutex);
+  if (val < 0)
+  {
+    val = -val;
+    assert(m_active_bound_threads_cnt >= (Uint32)val);
+    m_active_bound_threads_cnt -= val;
+  }
+  else
+  {
+    m_active_bound_threads_cnt += val;
+  }
+}
 
 void
 Ndbfs::report(Request * request, Signal* signal)
@@ -938,14 +1208,19 @@ Ndbfs::report(Request * request, Signal* signal)
   signal->setTrace(request->theTrace);
   const BlockReference ref = request->theUserReference;
 
-  if(!request->file->m_page_ptr.isNull())
+  if (request->file->has_buffer())
   {
-    assert(request->file->m_page_cnt > 0);
-    m_ctx.m_mm.release_pages(RT_DBTUP_PAGE, 
-                             request->file->m_page_ptr.i,
-                             request->file->m_page_cnt);
-    request->file->m_page_ptr.setNull();
-    request->file->m_page_cnt = 0;
+    if ((request->action == Request::open && request->error) ||
+        request->action == Request::close ||
+        request->action == Request::closeRemove ||
+        request->action == Request::buildindx)
+    {
+      Uint32 rg;
+      Uint32 cnt;
+      Ptr<GlobalPage> ptr;
+      request->file->clear_buffer(rg, ptr, cnt);
+      m_ctx.m_mm.release_pages(rg, ptr.i, cnt);
+    }
   }
   
   if (request->error) {
@@ -967,7 +1242,7 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request:: open: {
       jam();
       // Put the file back in idle files list
-      theIdleFiles.push_back(request->file);  
+      pushIdleFile(request->file);
       sendSignal(ref, GSN_FSOPENREF, signal, FsRef::SignalLength, JBB);
       break;
     }
@@ -975,6 +1250,14 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request:: close: {
       jam();
       sendSignal(ref, GSN_FSCLOSEREF, signal, FsRef::SignalLength, JBB);
+
+      g_eventLogger->warning("Error closing file: %s %u/%u",
+                             request->file->theFileName.c_str(),
+                             fsRef->errorCode,
+                             fsRef->osErrorCode);
+      g_eventLogger->warning("Dumping files");
+      signal->theData[0] = 405;
+      execDUMP_STATE_ORD(signal);
       break;
     }
     case Request:: writeSync:
@@ -1007,7 +1290,7 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request::rmrf: {
       jam();
       // Put the file back in idle files list
-      theIdleFiles.push_back(request->file);  
+      pushIdleFile(request->file);
       sendSignal(ref, GSN_FSREMOVEREF, signal, FsRef::SignalLength, JBB);
       break;
     }
@@ -1025,18 +1308,18 @@ Ndbfs::report(Request * request, Signal* signal)
       rep->errorCode = request->error;
       sendSignal(ref, GSN_ALLOC_MEM_REF, signal,
                  AllocMemRef::SignalLength, JBB);
-      theIdleFiles.push_back(request->file);
+      pushIdleFile(request->file);
       break;
     }
     case Request::buildindx: {
       jam();
-      BuildIndxRef* rep = (BuildIndxRef*)signal->getDataPtrSend();
-      rep->setUserRef(reference());
-      rep->setConnectionPtr(request->theUserPointer);
-      rep->setErrorCode((BuildIndxRef::ErrorCode)request->error);
-      sendSignal(ref, GSN_BUILDINDXREF, signal,
-                 BuildIndxRef::SignalLength, JBB);
-      theIdleFiles.push_back(request->file);
+      BuildIndxImplRef* rep = (BuildIndxImplRef*)signal->getDataPtrSend();
+      rep->senderRef = reference();
+      rep->senderData = request->theUserPointer;
+      rep->errorCode = (BuildIndxImplRef::ErrorCode)request->error;
+      sendSignal(ref, GSN_BUILD_INDX_IMPL_REF, signal,
+                 BuildIndxImplRef::SignalLength, JBB);
+      pushIdleFile(request->file);
       break;
     }
     }//switch
@@ -1054,6 +1337,7 @@ Ndbfs::report(Request * request, Signal* signal)
 	m_maxOpenedFiles = theOpenFiles.size();
 
       fsConf->filePointer = request->theFilePointer;
+      fsConf->fileInfo = request->m_fileinfo;
       sendSignal(ref, GSN_FSOPENCONF, signal, 3, JBA);
       break;
     }
@@ -1063,7 +1347,7 @@ Ndbfs::report(Request * request, Signal* signal)
       // removes the file from OpenFiles list
       theOpenFiles.erase(request->theFilePointer); 
       // Put the file in idle files list
-      theIdleFiles.push_back(request->file); 
+      pushIdleFile(request->file);
       sendSignal(ref, GSN_FSCLOSECONF, signal, 1, JBA);
       break;
     }
@@ -1083,7 +1367,7 @@ Ndbfs::report(Request * request, Signal* signal)
     }
     case Request:: readPartial: {
       jam();
-      fsConf->bytes_read = request->par.readWrite.pages[0].size;
+      fsConf->bytes_read = Uint32(request->par.readWrite.pages[0].size);
       sendSignal(ref, GSN_FSREADCONF, signal, 2, JBA);
       break;
     }
@@ -1096,14 +1380,14 @@ Ndbfs::report(Request * request, Signal* signal)
     case Request::append_synch:
     {
       jam();
-      signal->theData[1] = request->par.append.size;
+      signal->theData[1] = Uint32(request->par.append.size);
       sendSignal(ref, GSN_FSAPPENDCONF, signal, 2, JBA);
       break;
     }
     case Request::rmrf: {
       jam();
       // Put the file in idle files list
-      theIdleFiles.push_back(request->file);            
+      pushIdleFile(request->file);
       sendSignal(ref, GSN_FSREMOVECONF, signal, 1, JBA);
       break;
     }
@@ -1117,19 +1401,21 @@ Ndbfs::report(Request * request, Signal* signal)
       AllocMemConf* conf = (AllocMemConf*)signal->getDataPtrSend();
       conf->senderRef = reference();
       conf->senderData = request->theUserPointer;
+      conf->bytes_hi = Uint32(request->par.alloc.bytes >> 32);
+      conf->bytes_lo = Uint32(request->par.alloc.bytes);
       sendSignal(ref, GSN_ALLOC_MEM_CONF, signal,
                  AllocMemConf::SignalLength, JBB);
-      theIdleFiles.push_back(request->file);
+      pushIdleFile(request->file);
       break;
     }
     case Request::buildindx: {
       jam();
-      BuildIndxConf* rep = (BuildIndxConf*)signal->getDataPtrSend();
-      rep->setUserRef(reference());
-      rep->setConnectionPtr(request->theUserPointer);
-      sendSignal(ref, GSN_BUILDINDXCONF, signal,
-                 BuildIndxConf::SignalLength, JBB);
-      theIdleFiles.push_back(request->file);
+      BuildIndxImplConf* rep = (BuildIndxImplConf*)signal->getDataPtrSend();
+      rep->senderRef = reference();
+      rep->senderData = request->theUserPointer;
+      sendSignal(ref, GSN_BUILD_INDX_IMPL_CONF, signal,
+                 BuildIndxImplConf::SignalLength, JBB);
+      pushIdleFile(request->file);
       break;
     }
     }    
@@ -1329,9 +1615,13 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     infoEvent("NDBFS: Files: %d Open files: %d",
 	      theFiles.size(),
 	      theOpenFiles.size());
-    infoEvent(" Idle files: %d Max opened files: %d",
-	       theIdleFiles.size(),
-	       m_maxOpenedFiles);
+    infoEvent(" Idle files: %u Max opened files: %d",
+              theIdleFiles.size(),
+              m_maxOpenedFiles);
+    infoEvent(" Bound Threads: %u (active %u) Unbound threads: %u",
+              m_bound_threads_cnt,
+              m_active_bound_threads_cnt,
+              m_unbounds_threads_cnt);
     infoEvent(" Max files: %d",
 	      m_maxFiles);
     infoEvent(" Requests: %d",
@@ -1344,7 +1634,10 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     
     for (unsigned i = 0; i < theOpenFiles.size(); i++){
       AsyncFile* file = theOpenFiles.getFile(i);
-      infoEvent("%2d (0x%lx): %s", i, (long)file, file->theFileName.c_str());
+      infoEvent("%2d (0x%lx): %s thr: %lx", i,
+                (long)file,
+                file->theFileName.c_str(),
+                (long)file->getThread());
     }
     return;
   }
@@ -1358,17 +1651,20 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
     return;
   }
   if(signal->theData[0] == DumpStateOrd::NdbfsDumpIdleFiles){
-    infoEvent("NDBFS: Dump idle files: %d", theIdleFiles.size());
-    
+    infoEvent("NDBFS: Dump idle files: %u",
+              theIdleFiles.size());
+
     for (unsigned i = 0; i < theIdleFiles.size(); i++){
       AsyncFile* file = theIdleFiles[i];
       infoEvent("%2d (0x%lx): %s", i, (long)file, file->isOpen()?"OPEN":"CLOSED");
     }
+
     return;
   }
 
   if(signal->theData[0] == 404)
   {
+#if 0
     ndbrequire(signal->getLength() == 2);
     Uint32 file= signal->theData[1];
     AsyncFile* openFile = theOpenFiles.find(file);
@@ -1389,13 +1685,28 @@ Ndbfs::execDUMP_STATE_ORD(Signal* signal)
       AsyncFile* file = theFiles[i];
       ndbout_c("%2d (0x%lx): %s", i, (long) file, file->isOpen()?"OPEN":"CLOSED");
     }
+#endif
+  }
+
+  if(signal->theData[0] == 405)
+  {
+    for (unsigned i = 0; i < theFiles.size(); i++)
+    {
+      AsyncFile* file = theFiles[i];
+      if (file == 0)
+        continue;
+      ndbout_c("%u : %s %s fileInfo=%08x", i,
+               file->theFileName.c_str() ? file->theFileName.c_str() : "",
+               file->isOpen() ? "OPEN" : "CLOSED",
+               file->get_fileinfo());
+    }
   }
 }//Ndbfs::execDUMP_STATE_ORD()
 
 const char*
 Ndbfs::get_filename(Uint32 fd) const
 {
-  jamEntry();
+  jamNoBlock();
   const AsyncFile* openFile = theOpenFiles.find(fd);
   if(openFile)
     return openFile->theFileName.get_base_name();
@@ -1406,6 +1717,7 @@ Ndbfs::get_filename(Uint32 fd) const
 BLOCK_FUNCTIONS(Ndbfs)
 
 template class Vector<AsyncFile*>;
+template class Vector<AsyncIoThread*>;
 template class Vector<OpenFiles::OpenFileItem>;
 template class MemoryChannel<Request>;
 template class Pool<Request>;
