@@ -1,14 +1,21 @@
 /* 
-   Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2007, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -41,6 +48,13 @@
 #include <sys/uio.h>
 #include <dirent.h>
 
+#include <EventLogger.hpp>
+
+#define JAM_FILE_ID 384
+
+extern EventLogger* g_eventLogger;
+
+
 PosixAsyncFile::PosixAsyncFile(SimulatedBlock& fs) :
   AsyncFile(fs),
   theFd(-1),
@@ -76,9 +90,31 @@ int PosixAsyncFile::init()
 
   nzf.stream.opaque= &nz_mempool;
 
+  m_filetype = 0;
+
   return 0;
 }
 
+void PosixAsyncFile::set_or_check_filetype(bool set)
+{
+  struct stat sb;
+  if (fstat(theFd, &sb) == -1)
+  {
+    g_eventLogger->error("fd=%d: fstat errno=%d",
+                          theFd, errno);
+    abort();
+  }
+  int ft = sb.st_mode >> 12; // posix
+  if (set)
+    m_filetype = ft;
+  else if (m_filetype != ft)
+  {
+    g_eventLogger->error("fd=%d: type old=%d new=%d",
+                          theFd, m_filetype, ft);
+    abort();
+  }
+}
+
 #ifdef O_DIRECT
 static char g_odirect_readbuf[2*GLOBAL_PAGE_SIZE -1];
 #endif
@@ -192,10 +228,14 @@ void PosixAsyncFile::openReq(Request *request)
   }
 #endif
 
+  m_always_sync = false;
+
   if ((flags & FsOpenReq::OM_SYNC) && ! (flags & FsOpenReq::OM_INIT))
   {
 #ifdef O_SYNC
     new_flags |= O_SYNC;
+#else
+    m_always_sync = true;
 #endif
   }
 
@@ -334,7 +374,7 @@ no_odirect:
 
 #ifdef TRACE_INIT
     Uint32 write_cnt = 0;
-    Uint64 start = NdbTick_CurrentMillisecond();
+    const NDB_TICKS start = NdbTick_getCurrentTicks();
 #endif
     while(off < sz)
     {
@@ -403,8 +443,8 @@ no_odirect:
     }
     ::fsync(theFd);
 #ifdef TRACE_INIT
-    Uint64 stop = NdbTick_CurrentMillisecond();
-    Uint64 diff = stop - start;
+    const NDB_TICKS stop = NdbTick_getCurrentTicks();
+    Uint64 diff = NdbTick_Elapsed(start, stop).milliSec();
     if (diff == 0)
       diff = 1;
     ndbout_c("wrote %umb in %u writes %us -> %ukb/write %umb/s",
@@ -463,6 +503,8 @@ no_odirect:
     {
       request->error = errno;
     }
+#else
+    m_always_sync = true;
 #endif
   }
 
@@ -493,6 +535,9 @@ no_odirect:
       abort();
     }
   }
+
+  set_or_check_filetype(true);
+  request->m_fileinfo = get_fileinfo();
 }
 
 int PosixAsyncFile::readBuffer(Request *req, char *buf,
@@ -676,6 +721,7 @@ int PosixAsyncFile::writeBuffer(const char *buf, size_t size, off_t offset)
 
 void PosixAsyncFile::closeReq(Request *request)
 {
+  set_or_check_filetype(false);
   if (m_open_flags & (
       FsOpenReq::OM_WRITEONLY |
       FsOpenReq::OM_READWRITE |
@@ -715,7 +761,9 @@ bool PosixAsyncFile::isOpen(){
 
 void PosixAsyncFile::syncReq(Request *request)
 {
-  if(m_auto_sync_freq && m_write_wo_sync == 0){
+  if ((m_auto_sync_freq && m_write_wo_sync == 0) &&
+      !m_always_sync)
+  {
     return;
   }
   if (-1 == ::fsync(theFd)){
@@ -727,6 +775,7 @@ void PosixAsyncFile::syncReq(Request *request)
 
 void PosixAsyncFile::appendReq(Request *request)
 {
+  set_or_check_filetype(false);
   const char * buf = request->par.append.buf;
   Uint32 size = request->par.append.size;
 
@@ -756,7 +805,9 @@ void PosixAsyncFile::appendReq(Request *request)
     buf += n;
   }
 
-  if(m_auto_sync_freq && m_write_wo_sync > m_auto_sync_freq){
+  if ((m_auto_sync_freq && m_write_wo_sync > m_auto_sync_freq) ||
+      m_always_sync)
+  {
     syncReq(request);
   }
 }
