@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -24,19 +31,11 @@ Name:          Ndb.cpp
 
 #include <ndb_global.h>
 
-
-#include "NdbApiSignal.hpp"
-#include "NdbImpl.hpp"
-#include <NdbOperation.hpp>
-#include <NdbTransaction.hpp>
-#include <NdbEventOperation.hpp>
-#include <NdbEventOperationImpl.hpp>
-#include <NdbRecAttr.hpp>
+#include "API.hpp"
 #include <md5_hash.hpp>
 #include <NdbSleep.h>
 #include <NdbOut.hpp>
 #include <ndb_limits.h>
-#include "API.hpp"
 #include <NdbEnv.h>
 #include <BaseString.hpp>
 #include <NdbSqlUtil.hpp>
@@ -46,7 +45,7 @@ void connect();
 
 Connect to any node which has no connection at the moment.
 ****************************************************************************/
-NdbTransaction* Ndb::doConnect(Uint32 tConNode) 
+NdbTransaction* Ndb::doConnect(Uint32 tConNode, Uint32 instance)
 {
   Uint32        tNode;
   Uint32        tAnyAlive = 0;
@@ -55,12 +54,12 @@ NdbTransaction* Ndb::doConnect(Uint32 tConNode)
   DBUG_ENTER("Ndb::doConnect");
 
   if (tConNode != 0) {
-    TretCode = NDB_connect(tConNode);
+    TretCode = NDB_connect(tConNode, instance);
     if ((TretCode == 1) || (TretCode == 2)) {
 //****************************************************************************
 // We have connections now to the desired node. Return
 //****************************************************************************
-      DBUG_RETURN(getConnectedNdbTransaction(tConNode));
+      DBUG_RETURN(getConnectedNdbTransaction(tConNode, instance));
     } else if (TretCode < 0) {
       DBUG_RETURN(NULL);
     } else if (TretCode != 0) {
@@ -71,6 +70,7 @@ NdbTransaction* Ndb::doConnect(Uint32 tConNode)
 // We will connect to any node. Make sure that we have connections to all
 // nodes.
 //****************************************************************************
+  Uint32 anyInstance = 0;
   if (theImpl->m_optimized_node_selection)
   {
     Ndb_cluster_connection_node_iter &node_iter= 
@@ -78,14 +78,14 @@ NdbTransaction* Ndb::doConnect(Uint32 tConNode)
     theImpl->m_ndb_cluster_connection.init_get_next_node(node_iter);
     while ((tNode= theImpl->m_ndb_cluster_connection.get_next_node(node_iter)))
     {
-      TretCode= NDB_connect(tNode);
+      TretCode= NDB_connect(tNode, anyInstance);
       if ((TretCode == 1) ||
 	  (TretCode == 2))
       {
 //****************************************************************************
 // We have connections now to the desired node. Return
 //****************************************************************************
-	DBUG_RETURN(getConnectedNdbTransaction(tNode));
+	DBUG_RETURN(getConnectedNdbTransaction(tNode, anyInstance));
       } else if (TretCode < 0) {
         DBUG_RETURN(NULL);
       } else if (TretCode != 0) {
@@ -108,14 +108,14 @@ NdbTransaction* Ndb::doConnect(Uint32 tConNode)
 
       Tcount++;
       tNode= theImpl->theDBnodes[theCurrentConnectIndex];
-      TretCode= NDB_connect(tNode);
+      TretCode= NDB_connect(tNode, anyInstance);
       if ((TretCode == 1) ||
 	  (TretCode == 2))
       {
 //****************************************************************************
 // We have connections now to the desired node. Return
 //****************************************************************************
-	DBUG_RETURN(getConnectedNdbTransaction(tNode));
+	DBUG_RETURN(getConnectedNdbTransaction(tNode, anyInstance));
       } else if (TretCode < 0) {
         DBUG_RETURN(NULL);
       } else if (TretCode != 0) {
@@ -134,13 +134,20 @@ NdbTransaction* Ndb::doConnect(Uint32 tConNode)
 #endif
     theError.code = 4006;
   } else {
-    theError.code = 4009;
+    if (theImpl->m_transporter_facade->is_cluster_completely_unavailable())
+    {
+      theError.code = 4009;
+    }
+    else
+    {
+      theError.code = 4035;
+    }
   }//if
   DBUG_RETURN(NULL);
 }
 
 int 
-Ndb::NDB_connect(Uint32 tNode) 
+Ndb::NDB_connect(Uint32 tNode, Uint32 instance)
 {
 //****************************************************************************
 // We will perform seize of a transaction record in DBTC in the specified node.
@@ -151,18 +158,44 @@ Ndb::NDB_connect(Uint32 tNode)
   DBUG_ENTER("Ndb::NDB_connect");
 
   {
-    TransporterFacade *tp = theImpl->m_transporter_facade;
-    if (tp->get_node_stopping(tNode))
+    if (theImpl->get_node_stopping(tNode))
     {
       DBUG_RETURN(0);
     }
   }
 
   NdbTransaction * tConArray = theConnectionArray[tNode];
-  if (tConArray != NULL) {
+  if (instance != 0 && tConArray != 0)
+  {
+    NdbTransaction* prev = 0;
+    NdbTransaction* curr = tConArray;
+    while (curr)
+    {
+      if (refToInstance(curr->m_tcRef) == instance)
+      {
+        if (prev != 0)
+        {
+          prev->theNext = curr->theNext;
+          if (!curr->theNext)
+            theConnectionArrayLast[tNode] = prev;
+          curr->theNext = tConArray;
+          theConnectionArray[tNode] = curr;
+        }
+        else
+        {
+          assert(curr == tConArray);
+        }
+        DBUG_RETURN(2);
+      }
+      prev = curr;
+      curr = curr->theNext;
+    }
+  }
+  else if (tConArray != NULL)
+  {
     DBUG_RETURN(2);
   }
-  
+
   NdbTransaction * tNdbCon = getNdbCon();	// Get free connection object.
   if (tNdbCon == NULL) {
     DBUG_RETURN(4);
@@ -172,7 +205,7 @@ Ndb::NDB_connect(Uint32 tNode)
     releaseNdbCon(tNdbCon);
     DBUG_RETURN(4);
   }//if
-  if (tSignal->setSignal(GSN_TCSEIZEREQ) == -1) {
+  if (tSignal->setSignal(GSN_TCSEIZEREQ, DBTC) == -1) {
     releaseNdbCon(tNdbCon);
     releaseSignal(tSignal);
     DBUG_RETURN(4);
@@ -182,7 +215,9 @@ Ndb::NDB_connect(Uint32 tNode)
 // Set connection pointer as NdbTransaction object
 //************************************************
   tSignal->setData(theMyRef, 2);	// Set my block reference
+  tSignal->setData(instance, 3);        // Set requested instance
   tNdbCon->Status(NdbTransaction::Connecting); // Set status to connecting
+  tNdbCon->theDBnode = tNode;
   Uint32 nodeSequence;
   tReturnCode= sendRecSignal(tNode, WAIT_TC_SEIZE, tSignal,
                              0, &nodeSequence);
@@ -191,12 +226,10 @@ Ndb::NDB_connect(Uint32 tNode)
     //************************************************
     // Send and receive was successful
     //************************************************
-    NdbTransaction* tPrevFirst = theConnectionArray[tNode];
     tNdbCon->setConnectedNodeId(tNode, nodeSequence);
     
     tNdbCon->setMyBlockReference(theMyRef);
-    theConnectionArray[tNode] = tNdbCon;
-    tNdbCon->theNext = tPrevFirst;
+    prependConnectionArray(tNdbCon, tNode);
     DBUG_RETURN(1);
   } else {
 //****************************************************************************
@@ -229,9 +262,38 @@ Ndb::NDB_connect(Uint32 tNode)
 }//Ndb::NDB_connect()
 
 NdbTransaction *
-Ndb::getConnectedNdbTransaction(Uint32 nodeId){
+Ndb::getConnectedNdbTransaction(Uint32 nodeId, Uint32 instance){
   NdbTransaction* next = theConnectionArray[nodeId];
-  theConnectionArray[nodeId] = next->theNext;
+  if (instance != 0)
+  {
+    NdbTransaction * prev = 0;
+    while (next)
+    {
+      if (refToInstance(next->m_tcRef) == instance)
+      {
+        if (prev != 0)
+        {
+          assert(false); // Should have been moved in NDB_connect
+          prev->theNext = next->theNext;
+          if (!next->theNext)
+            theConnectionArrayLast[nodeId] = prev;
+          goto found_middle;
+        }
+        else
+        {
+          assert(next == theConnectionArray[nodeId]);
+          goto found_first;
+        }
+      }
+      prev = next;
+      next = next->theNext;
+    }
+    assert(false); // !!
+    return 0;
+  }
+found_first:
+  removeConnectionArray(next, nodeId);
+found_middle:
   next->theNext = NULL;
 
   return next;
@@ -308,7 +370,14 @@ Ndb::waitUntilReady(int timeout)
   if (theImpl->m_ndb_cluster_connection.wait_until_ready
       (timeout-secondsCounter,30) < 0)
   {
-    theError.code = 4009;
+    if (theImpl->m_transporter_facade->is_cluster_completely_unavailable())
+    {
+      theError.code = 4009;
+    }
+    else
+    {
+      theError.code = 4035;
+    }
     DBUG_RETURN(-1);
   }
 
@@ -335,6 +404,7 @@ Ndb::computeHash(Uint32 *retval,
   const NdbColumnImpl* const * cols = impl->m_columns.getBase();
   Uint32 len;
   char* pos;
+  void* malloced_buf = NULL;
 
   Uint32 colcnt = impl->m_columns.size();
   Uint32 parts = impl->m_noOfDistributionKeys;
@@ -371,6 +441,7 @@ Ndb::computeHash(Uint32 *retval,
       partcols[j++] = cols[i];
     }
   }
+  DBUG_ASSERT(j == parts);
 
   for (Uint32 i = 0; i<parts; i++)
   {
@@ -400,27 +471,31 @@ Ndb::computeHash(Uint32 *retval,
     sumlen += len;
 
   }
-  
-  if (buf)
+
+  if (!buf)
   {
+    bufLen = sumlen;
+    bufLen += sizeof(Uint64); /* add space for potential alignment */
+    buf = malloc(bufLen);
+    if (unlikely(buf == 0))
+      return 4000;
+    malloced_buf = buf; /* Remember to free */
+    assert(bufLen > sumlen);
+  }
+
+  {
+    /* Get 64-bit aligned ptr required for hashing */
+    assert(bufLen != 0);
     UintPtr org = UintPtr(buf);
     UintPtr use = (org + 7) & ~(UintPtr)7;
 
     buf = (void*)use;
-    bufLen -= (use - org);
+    bufLen -= Uint32(use - org);
 
     if (unlikely(sumlen > bufLen))
       goto ebuftosmall;
   }
-  else
-  {
-    buf = malloc(sumlen);
-    if (unlikely(buf == 0))
-      goto enomem;
-    bufLen = 0;
-    assert((UintPtr(buf) & 7) == 0);
-  }
-  
+
   pos = (char*)buf;
   for (Uint32 i = 0; i<parts; i++)
   {
@@ -465,7 +540,7 @@ Ndb::computeHash(Uint32 *retval,
       pos += len;
     }
   }
-  len = UintPtr(pos) - UintPtr(buf);
+  len = Uint32(UintPtr(pos) - UintPtr(buf));
   assert((len & 3) == 0);
 
   Uint32 values[4];
@@ -476,8 +551,8 @@ Ndb::computeHash(Uint32 *retval,
     * retval = values[1];
   }
   
-  if (bufLen == 0)
-    free(buf);
+  if (malloced_buf)
+    free(malloced_buf);
   
   return 0;
   
@@ -497,16 +572,175 @@ ebuftosmall:
   return 4278;
 
 emalformedstring:
-  if (bufLen == 0)
-    free(buf);
+  if (malloced_buf)
+    free(malloced_buf);
   
   return 4279;
   
 emalformedkey:
   return 4280;
+}
 
-enomem:
-  return 4000;
+int
+Ndb::computeHash(Uint32 *retval,
+                 const NdbRecord *keyRec,
+                 const char *keyData, 
+                 void* buf, Uint32 bufLen)
+{
+  Uint32 len;
+  char* pos = NULL;
+  void* malloced_buf = NULL;
+
+  Uint32 parts = keyRec->distkey_index_length;
+
+  if (unlikely(keyRec->flags & NdbRecord::RecHasUserDefinedPartitioning))
+  {
+    /* Calculating native hash on keys in user defined 
+     * partitioned table is probably part of a bug
+     */
+    goto euserdeftable;
+  }
+
+  if (!buf)
+  {
+    /* We malloc buf here.  Don't have a handy 'Max distr key size'
+     * variable, so let's use the key length, which must include
+     * the Distr key.
+     */
+    bufLen = keyRec->m_keyLenInWords << 2;
+    bufLen += sizeof(Uint64); /* add space for potential alignment */
+    buf = malloc(bufLen);
+    if (unlikely(buf == 0))
+      return 4000;
+    malloced_buf = buf; /* Remember to free */
+  }
+
+  {
+    /* Get 64-bit aligned address as required for hashing */
+    assert(bufLen != 0);
+    UintPtr org = UintPtr(buf);
+    UintPtr use = (org + 7) & ~(UintPtr)7;
+
+    buf = (void*)use;
+    bufLen -= Uint32(use - org);
+  }
+
+  pos= (char*) buf;
+
+  for (Uint32 i = 0; i < parts; i++)
+  {
+    const struct NdbRecord::Attr &keyAttr =
+      keyRec->columns[keyRec->distkey_indexes[i]];
+
+    Uint32 len;
+    Uint32 maxlen = keyAttr.maxSize;
+    unsigned char *src= (unsigned char*)keyData + keyAttr.offset;
+
+    if (keyAttr.flags & NdbRecord::IsVar1ByteLen)
+    {
+      if (keyAttr.flags & NdbRecord::IsMysqldShrinkVarchar)
+      {
+        len = uint2korr(src);
+        src += 2;
+      }
+      else
+      {
+        len = *src;
+        src += 1;
+      }
+      maxlen -= 1;
+    }
+    else if (keyAttr.flags & NdbRecord::IsVar2ByteLen)
+    {
+      len = uint2korr(src);
+      src += 2;
+      maxlen -= 2;
+    }
+    else
+    {
+      len = maxlen;
+    }
+
+    const CHARSET_INFO* cs = keyAttr.charset_info;
+    if (cs)
+    {
+      Uint32 xmul = cs->strxfrm_multiply;
+      if (xmul == 0)
+        xmul = 1;
+      /*
+       * Varchar end-spaces are ignored in comparisons.  To get same hash
+       * we blank-pad to maximum length via strnxfrm.
+       */
+      Uint32 dstLen = xmul * maxlen;
+      int n = NdbSqlUtil::strnxfrm_bug7284((CHARSET_INFO*)cs,
+                                           (unsigned char*)pos,
+                                           dstLen, src,
+                                           len);
+      if (unlikely(n == -1))
+        goto emalformedstring;
+      len = n;
+    }
+    else
+    {
+      if (keyAttr.flags & NdbRecord::IsVar1ByteLen)
+      {
+        *pos= (unsigned char)len;
+        memcpy(pos+1, src, len);
+        len += 1;
+      }
+      else if (keyAttr.flags & NdbRecord::IsVar2ByteLen)
+      {
+        len += 2;
+        memcpy(pos, src-2, len);
+      }
+      else
+        memcpy(pos, src, len);
+    }
+    while (len & 3)
+    {
+      * (pos + len++) = 0;
+    }
+    pos += len;
+  }
+  len = Uint32(UintPtr(pos) - UintPtr(buf));
+  assert((len & 3) == 0);
+
+  Uint32 values[4];
+  md5_hash(values, (const Uint64*)buf, len >> 2);
+  
+  if (retval)
+  {
+    * retval = values[1];
+  }
+
+  if (malloced_buf)
+    free(malloced_buf);
+
+  return 0;
+
+euserdeftable:
+  return 4544;
+
+  
+emalformedstring:
+  if (malloced_buf)
+    free(malloced_buf);
+
+  return 4279;
+}
+
+NdbTransaction*
+Ndb::startTransaction(const NdbRecord *keyRec, const char *keyData,
+                      void* xfrmbuf, Uint32 xfrmbuflen)
+{
+  int ret;
+  Uint32 hash;
+  if ((ret = computeHash(&hash, keyRec, keyData, xfrmbuf, xfrmbuflen)) == 0)
+  {
+    return startTransaction(keyRec->table, keyRec->table->getPartitionId(hash));
+  }
+  theError.code = ret;
+  return 0;
 }
 
 NdbTransaction* 
@@ -526,7 +760,8 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
 }
 
 NdbTransaction*
-Ndb::startTransaction(const NdbDictionary::Table* table, Uint32 partitionId)
+Ndb::startTransaction(const NdbDictionary::Table* table,
+                      Uint32 partitionId)
 {
   DBUG_ENTER("Ndb::startTransaction");
   DBUG_PRINT("enter", 
@@ -545,7 +780,32 @@ Ndb::startTransaction(const NdbDictionary::Table* table, Uint32 partitionId)
     else
       nodeId= 0;
     
-    NdbTransaction *trans= startTransactionLocal(0, nodeId);
+    theImpl->incClientStat(TransStartCount, 1);
+
+    NdbTransaction *trans= startTransactionLocal(0, nodeId, 0);
+    DBUG_PRINT("exit",("start trans: 0x%lx  transid: 0x%lx",
+                       (long) trans,
+                       (long) (trans ? trans->getTransactionId() : 0)));
+    DBUG_RETURN(trans);
+  }
+  DBUG_RETURN(NULL);
+}
+
+NdbTransaction*
+Ndb::startTransaction(Uint32 nodeId,
+                      Uint32 instanceId)
+{
+  DBUG_ENTER("Ndb::startTransaction");
+  DBUG_PRINT("enter", 
+             ("nodeId: %u instanceId: %u", nodeId, instanceId));
+  if (theInitState == Initialised) 
+  {
+    theError.code = 0;
+    checkFailedNode();
+
+    theImpl->incClientStat(TransStartCount, 1);
+
+    NdbTransaction *trans= startTransactionLocal(0, nodeId, instanceId);
     DBUG_PRINT("exit",("start trans: 0x%lx  transid: 0x%lx",
                        (long) trans,
                        (long) (trans ? trans->getTransactionId() : 0)));
@@ -608,8 +868,13 @@ Ndb::startTransaction(const NdbDictionary::Table *table,
       }
     }
 
+    /* TODO : Should call method above rather than duplicate call to
+     * startTransactionLocal
+     */
+    theImpl->incClientStat( TransStartCount, 1 );
+
     {
-      NdbTransaction *trans= startTransactionLocal(0, nodeId);
+      NdbTransaction *trans= startTransactionLocal(0, nodeId, 0);
       DBUG_PRINT("exit",("start trans: 0x%lx  transid: 0x%lx",
 			 (long) trans,
                          (long) (trans ? trans->getTransactionId() : 0)));
@@ -645,7 +910,9 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
     checkFailedNode();
 
     Uint32 nodeId = pBuddyTrans->getConnectedNodeId();
-    NdbTransaction* pCon = startTransactionLocal(aPriority, nodeId);
+    NdbTransaction* pCon =
+      startTransactionLocal(aPriority, nodeId,
+                            refToInstance(pBuddyTrans->m_tcRef));
     if(pCon == NULL)
       DBUG_RETURN(NULL);
 
@@ -653,6 +920,7 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
       // We could not get a connection to the desired node
       // release the connection and return NULL
       closeTransaction(pCon);
+      theImpl->decClientStat( TransStartCount, 1 ); /* Correct stats */
       theError.code = 4006;
       DBUG_RETURN(NULL);
     }
@@ -667,8 +935,8 @@ Ndb::hupp(NdbTransaction* pBuddyTrans)
   }//if
 }//Ndb::hupp()
 
-NdbTransaction* 
-Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId)
+NdbTransaction*
+Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId, Uint32 instance)
 {
 #ifdef VM_TRACE
   char buf[255];
@@ -689,7 +957,7 @@ Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId)
   
   NdbTransaction* tConnection;
   Uint64 tFirstTransId = theFirstTransId;
-  tConnection = doConnect(nodeId);
+  tConnection = doConnect(nodeId, instance);
   if (tConnection == NULL) {
     DBUG_RETURN(NULL);
   }//if
@@ -723,6 +991,48 @@ Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId)
   DBUG_RETURN(tConnection);
 }//Ndb::startTransactionLocal()
 
+void
+Ndb::appendConnectionArray(NdbTransaction *aCon, Uint32 nodeId)
+{
+  NdbTransaction *last = theConnectionArrayLast[nodeId];
+  if (last)
+  {
+    last->theNext = aCon;
+  }
+  else
+  {
+    theConnectionArray[nodeId] = aCon;
+  }
+  aCon->theNext = NULL;
+  theConnectionArrayLast[nodeId] = aCon;
+}
+
+void
+Ndb::prependConnectionArray(NdbTransaction *aCon, Uint32 nodeId)
+{
+  NdbTransaction *first = theConnectionArray[nodeId];
+  aCon->theNext = first;
+  if (!first)
+  {
+    theConnectionArrayLast[nodeId] = aCon;
+  }
+  theConnectionArray[nodeId] = aCon;
+}
+
+void
+Ndb::removeConnectionArray(NdbTransaction *first, Uint32 nodeId)
+{
+  NdbTransaction *next = first->theNext;
+  if (!next)
+  {
+    theConnectionArray[nodeId] = theConnectionArrayLast[nodeId] = NULL;
+  }
+  else
+  {
+    theConnectionArray[nodeId] = next;
+  }
+}
+
 /*****************************************************************************
 void closeTransaction(NdbTransaction* aConnection);
 
@@ -751,6 +1061,11 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   tCon = theTransactionList;
   theRemainingStartTransactions++;
   
+  DBUG_EXECUTE_IF("ndb_delay_close_txn", {
+    fprintf(stderr, "Ndb::closeTransaction() (%p) taking a break\n", this);
+    NdbSleep_MilliSleep(1000);
+    fprintf(stderr, "Ndb::closeTransaction() resuming\n");
+  });
   DBUG_PRINT("info",("close trans: 0x%lx  transid: 0x%lx",
                      (long) aConnection,
                      (long) aConnection->getTransactionId()));
@@ -794,6 +1109,8 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   
   aConnection->release();
   
+  theImpl->incClientStat(TransCloseCount, 1);
+
   if(aConnection->theError.code == 4008){
     /**
      * Something timed-out, returning the NdbTransaction leads
@@ -810,8 +1127,7 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
    * NOTE: It's ok to call getNodeSequence() here wo/ having mutex,
    */
   Uint32 nodeId = aConnection->getConnectedNodeId();
-  TransporterFacade* tp = theImpl->m_transporter_facade;   
-  Uint32 seq = tp->getNodeSequence(nodeId);
+  Uint32 seq = theImpl->getNodeSequence(nodeId);
   if (aConnection->theNodeSequence != seq)
   {
     aConnection->theReleaseOnClose = true;
@@ -822,8 +1138,8 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
     /**
      * Put it back in idle list for that node
      */
-    aConnection->theNext = theConnectionArray[nodeId];
-    theConnectionArray[nodeId] = aConnection;
+    appendConnectionArray(aConnection, nodeId);
+
     DBUG_VOID_RETURN;
   } else {
     aConnection->theReleaseOnClose = false;
@@ -832,171 +1148,6 @@ Ndb::closeTransaction(NdbTransaction* aConnection)
   DBUG_VOID_RETURN;
 }//Ndb::closeTransaction()
 
-/*****************************************************************************
-int* NdbTamper(int aAction, int aNode);
-
-Parameters: aAction     Specifies what action to be taken
-            1: Lock global checkpointing    Can only be sent to master DIH, Parameter aNode ignored.
-            2: UnLock global checkpointing    Can only be sent to master DIH, Parameter aNode ignored.
-	    3: Crash node
-
-           aNode        Specifies which node the action will be taken
-     	  -1: Master DIH 
-       	0-16: Nodnumber
-
-Return Value: -1 Error  .
-                
-Remark:         Sends a signal to DIH.
-*****************************************************************************/ 
-int 
-Ndb::NdbTamper(TamperType aAction, int aNode)
-{
-  NdbTransaction*	tNdbConn;
-  NdbApiSignal		tSignal(theMyRef);
-  int			tNode;
-  int                   tAction;
-  int			ret_code;
-
-#ifdef CUSTOMER_RELEASE
-  return -1;
-#else
-  DBUG_ENTER("Ndb::NdbTamper");
-  CHECK_STATUS_MACRO;
-  checkFailedNode();
-
-  theRestartGCI = 0;
-  switch (aAction) {
-// Translate enum to integer. This is done because the SCI layer
-// expects integers. 
-     case LockGlbChp:
-        tAction = 1;
-        break;
-     case UnlockGlbChp:
-        tAction = 2;
-	break;
-     case CrashNode:
-        tAction = 3;
-        break;
-     case ReadRestartGCI:
-	tAction = 4;
-	break;
-     default:
-        theError.code = 4102;
-        DBUG_RETURN(-1);
-  }
-
-  tNdbConn = getNdbCon();	// Get free connection object
-  if (tNdbConn == NULL) {
-    theError.code = 4000;
-    DBUG_RETURN(-1);
-  }
-  tSignal.setSignal(GSN_DIHNDBTAMPER);
-  tSignal.setData (tAction, 1);
-  tSignal.setData(tNdbConn->ptr2int(),2);
-  tSignal.setData(theMyRef,3);		// Set return block reference
-  tNdbConn->Status(NdbTransaction::Connecting); // Set status to connecting
-  TransporterFacade *tp = theImpl->m_transporter_facade;
-  if (tAction == 3) {
-    tp->lock_mutex();
-    tp->sendSignal(&tSignal, aNode);
-    tp->unlock_mutex();
-    releaseNdbCon(tNdbConn);
-  } else if ( (tAction == 2) || (tAction == 1) ) {
-    tp->lock_mutex();
-    tNode = tp->get_an_alive_node();
-    if (tNode == 0) {
-      theError.code = 4002;
-      releaseNdbCon(tNdbConn);
-      DBUG_RETURN(-1);
-    }//if
-    ret_code = tp->sendSignal(&tSignal,aNode);
-    tp->unlock_mutex();
-    releaseNdbCon(tNdbConn);
-    DBUG_RETURN(ret_code);
-  } else {
-    do {
-      tp->lock_mutex();
-      // Start protected area
-      tNode = tp->get_an_alive_node();
-      tp->unlock_mutex();
-      // End protected area
-      if (tNode == 0) {
-        theError.code = 4009;
-        releaseNdbCon(tNdbConn);
-        DBUG_RETURN(-1);
-      }//if
-      ret_code = sendRecSignal(tNode, WAIT_NDB_TAMPER, &tSignal, 0);
-      if (ret_code == 0) {  
-        if (tNdbConn->Status() != NdbTransaction::Connected) {
-          theRestartGCI = 0;
-        }//if
-        releaseNdbCon(tNdbConn);
-        DBUG_RETURN(theRestartGCI);
-      } else if ((ret_code == -5) || (ret_code == -2)) {
-        TRACE_DEBUG("Continue DIHNDBTAMPER when node failed/stopping");
-      } else {
-        DBUG_RETURN(-1);
-      }//if
-    } while (1);
-  }
-  DBUG_RETURN(0);
-#endif
-}
-#if 0
-/****************************************************************************
-NdbSchemaCon* startSchemaTransaction();
-
-Return Value:   Returns a pointer to a schema connection object.
-                Return NULL otherwise.
-Remark:         Start schema transaction. Synchronous.
-****************************************************************************/ 
-NdbSchemaCon* 
-Ndb::startSchemaTransaction()
-{
-  NdbSchemaCon* tSchemaCon;
-  if (theSchemaConToNdbList != NULL) {
-    theError.code = 4321;
-    return NULL;
-  }//if
-  tSchemaCon = new NdbSchemaCon(this);
-  if (tSchemaCon == NULL) {
-    theError.code = 4000;
-    return NULL;
-  }//if 
-  theSchemaConToNdbList = tSchemaCon;
-  return tSchemaCon;  
-}
-/*****************************************************************************
-void closeSchemaTransaction(NdbSchemaCon* aSchemaCon);
-
-Parameters:     aSchemaCon: the schemacon used in the transaction.
-Remark:         Close transaction by releasing the schemacon and all schemaop.
-*****************************************************************************/
-void
-Ndb::closeSchemaTransaction(NdbSchemaCon* aSchemaCon)
-{
-  if (theSchemaConToNdbList != aSchemaCon) {
-    abort();
-    return;
-  }//if
-  aSchemaCon->release();
-  delete aSchemaCon;
-  theSchemaConToNdbList = NULL;
-  return;
-}//Ndb::closeSchemaTransaction()
-#endif
-
-/*****************************************************************************
-void RestartGCI(int aRestartGCI);
-
-Remark:		Set theRestartGCI on the NDB object
-*****************************************************************************/
-void
-Ndb::RestartGCI(int aRestartGCI)
-{
-  theRestartGCI = aRestartGCI;
-}
-
 /****************************************************************************
 int getBlockNumber(void);
 
@@ -1120,51 +1271,76 @@ Ndb::getTupleIdFromNdb(const NdbTableImpl* table,
   5,15,25,35,...  
 */
   DBUG_ENTER("Ndb::getTupleIdFromNdb");
+  DBUG_PRINT("info", ("range.first_id=%llu, last_id=%llu, highest_seen=%llu "
+                      "tupleId = %llu, cacheSize=%u step=%llu start=%llu",
+                      range.m_first_tuple_id,
+                      range.m_last_tuple_id,
+                      range.m_highest_seen,
+                      tupleId,
+                      cacheSize,
+                      step,
+                      start));
+
   /*
-   Check if the next value can be taken from the pre-fetched
-   sequence.
+    If start value is greater than step it is ignored
   */
-  if (range.m_first_tuple_id != range.m_last_tuple_id &&
-      range.m_first_tuple_id + step <= range.m_last_tuple_id)
+  Uint64 offset = (start > step) ? 1 : start;
+
+  if (range.m_first_tuple_id != range.m_last_tuple_id)
   {
-    assert(range.m_first_tuple_id < range.m_last_tuple_id);
-    range.m_first_tuple_id += step; 
-    tupleId = range.m_first_tuple_id;
-    DBUG_PRINT("info", ("Next cached value %lu", (ulong) tupleId));
+    /**
+     * Range is valid and has span
+     * Determine next value *after* m_first_tuple_id
+     * meeting start and step constraints, then see
+     * if it is inside the cached range.
+     * m_first_tuple_id start may not meet the constraints 
+     * (if there was a manual insert)
+     * c.f. handler.cc compute_next_insert_id()
+     */
+    assert(step > 0);
+    assert(range.m_first_tuple_id >= offset);
+    Uint64 desiredNextVal = 0;
+    Uint64 numStepsTaken = ((range.m_first_tuple_id - offset) /
+                            step);
+    desiredNextVal = ((numStepsTaken + 1) * step) + offset;
+    DBUG_PRINT("info", ("desiredNextVal = %llu", desiredNextVal));
+
+    if (desiredNextVal <= range.m_last_tuple_id)
+    {
+      DBUG_PRINT("info", ("Next value from cache %lu", (ulong) tupleId));
+      assert(range.m_first_tuple_id < range.m_last_tuple_id);
+      range.m_first_tuple_id = tupleId = desiredNextVal; 
+      DBUG_RETURN(0);
+    }
   }
-  else
-  {
-    /*
-      If start value is greater than step it is ignored
-     */
-    Uint64 offset = (start > step) ? 1 : start;
+  
+  /*
+    Pre-fetch a number of values depending on cacheSize
+  */
+  if (cacheSize == 0)
+    cacheSize = 1;
+  
+  DBUG_PRINT("info", ("reading %u values from database", (uint)cacheSize));
+  /*
+   * reserve next cacheSize entries in db.  adds cacheSize to NEXTID
+   * and returns first tupleId in the new range. If tupleId's are
+   * incremented in steps then multiply the cacheSize with step size.
+   */
+  Uint64 opValue = cacheSize * step;
+  
+  if (opTupleIdOnNdb(table, range, opValue, 0) == -1)
+    DBUG_RETURN(-1);
+  DBUG_PRINT("info", ("Next value fetched from database %lu", (ulong) opValue));
+  DBUG_PRINT("info", ("Increasing %lu by offset %lu, increment  is %lu", 
+                      (ulong) (ulong) opValue, (ulong) offset, (ulong) step));
+  Uint64 current, next;
+  Uint64 div = ((Uint64) (opValue + step - offset)) / step;
+  next = div * step + offset;
+  current = (next < step) ? next : next - step;
+  tupleId = (opValue <= current) ? current : next;
+  DBUG_PRINT("info", ("Returning %lu", (ulong) tupleId));
+  range.m_first_tuple_id = tupleId;
 
-    /*
-      Pre-fetch a number of values depending on cacheSize
-     */
-    if (cacheSize == 0)
-      cacheSize = 1;
-
-    DBUG_PRINT("info", ("reading %u values from database", (uint)cacheSize));
-    /*
-     * reserve next cacheSize entries in db.  adds cacheSize to NEXTID
-     * and returns first tupleId in the new range. If tupleId's are
-     * incremented in steps then multiply the cacheSize with step size.
-     */
-    Uint64 opValue = cacheSize * step;
-
-    if (opTupleIdOnNdb(table, range, opValue, 0) == -1)
-      DBUG_RETURN(-1);
-    DBUG_PRINT("info", ("Next value fetched from database %lu", (ulong) opValue));
-    DBUG_PRINT("info", ("Increasing %lu by offset %lu, increment  is %lu", (ulong) (ulong) opValue, (ulong) offset, (ulong) step));
-    Uint64 current, next;
-    Uint64 div = ((Uint64) (opValue + step - offset)) / step;
-    next = div * step + offset;
-    current = (next < step) ? next : next - step;
-    tupleId = (opValue <= current) ? current : next;
-    DBUG_PRINT("info", ("Returning %lu", (ulong) tupleId));
-    range.m_first_tuple_id = tupleId;
-  }
   DBUG_RETURN(0);
 }
 
@@ -1333,6 +1509,12 @@ Ndb::setTupleIdInNdb(const NdbTableImpl* table,
                      TupleIdRange & range, Uint64 tupleId, bool modify)
 {
   DBUG_ENTER("Ndb::setTupleIdInNdb");
+  DBUG_PRINT("info", ("range first : %llu, last : %llu, tupleId : %llu "
+                      "modify %u",
+                      range.m_first_tuple_id,
+                      range.m_last_tuple_id,
+                      tupleId,
+                      modify));
   if (modify)
   {
     if (checkTupleIdInNdb(range, tupleId))
@@ -1348,6 +1530,11 @@ Ndb::setTupleIdInNdb(const NdbTableImpl* table,
           DBUG_PRINT("info", 
                      ("Setting next auto increment cached value to %lu",
                       (ulong)tupleId));  
+          DBUG_PRINT("info", 
+                     ("Range.m_first = %llu, m_last=%llu, m_highest_seen=%llu",
+                      range.m_first_tuple_id,
+                      range.m_last_tuple_id,
+                      range.m_highest_seen));
           DBUG_RETURN(0);
         }
       }
@@ -1452,6 +1639,7 @@ Ndb::opTupleIdOnNdb(const NdbTableImpl* table,
   tConnection = this->startTransaction(m_sys_tab_0,
                                        (const char *) &aTableId,
                                        sizeof(Uint32));
+
   if (tConnection == NULL)
     goto error_handler;
 
@@ -1615,6 +1803,29 @@ int Ndb::setSchemaName(const char * a_schema_name)
 }
 // </internal>
  
+const char* Ndb::getNdbObjectName() const
+{
+  return theImpl->m_ndbObjectName.c_str();
+}
+
+int Ndb::setNdbObjectName(const char *name)
+{
+  if (!theImpl->m_ndbObjectName.empty())
+  {
+    theError.code = 4121;
+    return -1; // Cannot set twice
+  }
+
+  if (theInitState != NotInitialised)
+  {
+    theError.code = 4122;
+    return -1; // Should be set before init() is called
+  }
+
+  theImpl->m_ndbObjectName.assign(name);
+  return 0;
+}
+
 const char * Ndb::getDatabaseName() const
 {
   return getCatalogName();
@@ -1863,7 +2074,20 @@ Ndb::getSchemaFromInternalName(const char * internalName)
   return ret;
 }
 
-// ToDo set event buffer size
+unsigned Ndb::get_eventbuf_max_alloc()
+{
+    return theEventBuffer->m_max_alloc;
+}
+
+void
+Ndb::set_eventbuf_max_alloc(unsigned sz)
+{
+  if (theEventBuffer != NULL)
+  {
+    theEventBuffer->m_max_alloc = sz;
+  }
+}
+
 NdbEventOperation* Ndb::createEventOperation(const char* eventName)
 {
   DBUG_ENTER("Ndb::createEventOperation");
@@ -1909,7 +2133,21 @@ NdbEventOperation *Ndb::getEventOperation(NdbEventOperation* tOp)
 int
 Ndb::pollEvents(int aMillisecondNumber, Uint64 *latestGCI)
 {
-  return theEventBuffer->pollEvents(aMillisecondNumber, latestGCI);
+  /* Look for already available events without polling transporter. */
+  const int found = theEventBuffer->pollEvents(latestGCI);
+  if (found)
+    return found;
+
+  /**
+   * We need to poll the transporter, and possibly wait, to make sure
+   * that arrived events are delivered to their clients as soon as possible.
+   * ::trp_deliver_signal() will wakeup the client when event arrives.
+   */
+  PollGuard poll_guard(* theImpl);
+  poll_guard.wait_n_unlock(aMillisecondNumber, 0, WAIT_EVENT);
+  // PollGuard ends here
+
+  return theEventBuffer->pollEvents(latestGCI);
 }
 
 int
@@ -1940,9 +2178,16 @@ Ndb::isConsistentGCI(Uint64 gci)
 
 const NdbEventOperation*
 Ndb::getGCIEventOperations(Uint32* iter, Uint32* event_types)
+{
+  return getNextEventOpInEpoch3(iter, event_types, NULL);
+}
+
+const NdbEventOperation*
+Ndb::getNextEventOpInEpoch3(Uint32* iter, Uint32* event_types,
+                           Uint32* cumulative_any_value)
 {
   NdbEventOperationImpl* op =
-    theEventBuffer->getGCIEventOperations(iter, event_types);
+    theEventBuffer->getEpochEventOperations(iter, event_types, cumulative_any_value);
   if (op != NULL)
     return op->m_facade;
   return NULL;
@@ -1971,6 +2216,19 @@ void Ndb::setReportThreshEventFreeMem(unsigned thresh)
   }
 }
 
+Uint64 Ndb::allocate_transaction_id()
+{
+  Uint64 ret= theFirstTransId;
+
+  if ((theFirstTransId & 0xFFFFFFFF) == 0xFFFFFFFF) {
+    theFirstTransId = (theFirstTransId >> 32) << 32;
+  } else {
+    theFirstTransId++;
+  }
+
+  return ret;
+}
+
 #ifdef VM_TRACE
 #include <NdbMutex.h>
 extern NdbMutex *ndb_print_state_mutex;
@@ -2183,11 +2441,80 @@ Ndb::getNdbErrorDetail(const NdbError& err, char* buff, Uint32 buffLen) const
 void
 Ndb::setCustomData(void* _customDataPtr)
 {
-  theImpl->customDataPtr = _customDataPtr;
+  theImpl->customData = Uint64(_customDataPtr);
 }
 
 void*
 Ndb::getCustomData() const
 {
-  return theImpl->customDataPtr;
+  return (void*)theImpl->customData;
+}
+
+void
+Ndb::setCustomData64(Uint64 _customData)
+{
+  theImpl->customData = _customData;
+}
+
+Uint64
+Ndb::getCustomData64() const
+{
+  return theImpl->customData;
+}
+
+Uint64
+Ndb::getNextTransactionId() const
+{
+  return theFirstTransId;
+}
+
+Uint32
+Ndb::getMinDbNodeVersion() const
+{
+  return theCachedMinDbNodeVersion;
+}
+
+const char* ClientStatNames [] =
+{ "WaitExecCompleteCount",
+  "WaitScanResultCount",
+  "WaitMetaRequestCount",
+  "WaitNanosCount",
+  "BytesSentCount",
+  "BytesRecvdCount",
+  "TransStartCount",
+  "TransCommitCount",
+  "TransAbortCount",
+  "TransCloseCount",
+  "PkOpCount",
+  "UkOpCount",
+  "TableScanCount",
+  "RangeScanCount",
+  "PrunedScanCount",
+  "ScanBatchCount",
+  "ReadRowCount",
+  "TransLocalReadRowCount",
+  "DataEventsRecvdCount",
+  "NonDataEventsRecvdCount",
+  "EventBytesRecvdCount",
+  "ForcedSendsCount",
+  "UnforcedSendsCount",
+  "DeferredSendsCount"
+};
+
+Uint64
+Ndb::getClientStat(Uint32 id) const
+{
+  if (likely(id < NumClientStatistics))
+    return theImpl->clientStats[id];
+
+  return 0;
+}
+
+const char*
+Ndb::getClientStatName(Uint32 id) const
+{
+  if (likely(id < NumClientStatistics))
+    return ClientStatNames[id];
+
+  return NULL;
 }
