@@ -1,19 +1,26 @@
-/* Copyright (c) 2003-2007 MySQL AB
-
+/*
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef NDBT_RESTARTER_HPP
 #define NDBT_RESTARTER_HPP
@@ -82,9 +89,11 @@ public:
 
   int getNumDbNodes();
   int insertErrorInNode(int _nodeId, int error);
+  int insertErrorInNodes(const int * _nodes, int _num_nodes, int error);
   int insertErrorInAllNodes(int error);
 
   int insertError2InNode(int _nodeId, int error, int extra);
+  int insertError2InNodes(const int * _nodes, int _num_nodes, int error, int extra);
   int insertError2InAllNodes(int error, int extra);
 
   int enterSingleUserMode(int _nodeId);
@@ -96,6 +105,7 @@ public:
   int getMasterNodeId();
   int getNextMasterNodeId(int nodeId);
   int getNodeGroup(int nodeId);
+  int getNodeGroups(Vector<int>& node_groups, int * max_alive_replicas_ptr = NULL);
   int getRandomNodeSameNodeGroup(int nodeId, int randomNumber);
   int getRandomNodeOtherNodeGroup(int nodeId, int randomNumber);
   int getRandomNotMasterNodeId(int randomNumber);
@@ -125,6 +135,8 @@ public:
   void setReconnect(bool);
 
   int rollingRestart(Uint32 flags = 0);
+
+  int getNodeConnectCount(int nodeId);
 protected:
 
   int waitClusterState(ndb_mgm_node_status _status,
