@@ -1,19 +1,26 @@
-/* Copyright (c) 2003-2005 MySQL AB
-
+/*
+   Copyright (c) 2004, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA */
-
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #include <NDBT_Test.hpp>
 #include <NDBT_ReturnCodes.h>
@@ -156,19 +163,26 @@ static
 int
 setupUDPartitioning(Ndb* ndb, NdbDictionary::Table& tab)
 {
-  /* Following should really be taken from running test system : */
-  const Uint32 numNodes= ndb->get_ndb_cluster_connection().no_db_nodes();
-  const Uint32 numReplicas= 2; // Assumption
-  const Uint32 guessNumNgs= numNodes/2;
-  const Uint32 numNgs= guessNumNgs?guessNumNgs : 1;
-  const Uint32 numFragsPerNode= 2 + (rand() % 3);
-  const Uint32 numPartitions= numReplicas * numNgs * numFragsPerNode;
+  NdbRestarter restarter;
+  Vector<int> node_groups;
+  int max_alive_replicas;
+  if (restarter.getNodeGroups(node_groups, &max_alive_replicas) == -1)
+  {
+    return -1;
+  }
+
+  const Uint32 numNgs = node_groups.size();
+
+  // Assume at least one node group had all replicas alive.
+  const Uint32 numReplicas = max_alive_replicas;
+  const Uint32 numFragsPerNode = 2 + (rand() % 3);
+  const Uint32 numPartitions = numReplicas * numNgs * numFragsPerNode;
 
   tab.setFragmentType(NdbDictionary::Table::UserDefined);
   tab.setFragmentCount(numPartitions);
-  for (Uint32 i=0; i<numPartitions; i++)
+  for (Uint32 i = 0; i < numPartitions; i++)
   {
-    frag_ng_mappings[i]= i % numNgs;
+    frag_ng_mappings[i] = node_groups[i % numNgs];
   }
   tab.setFragmentData(frag_ng_mappings, numPartitions);
 
