@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -1941,7 +1948,6 @@ runBug13394788(NDBT_Context* ctx, NDBT_Step* step)
  */
 namespace TupErr
 {
-  static const char* const tabName = "tupErrTab";
   static const int totalRowCount = 2000;
   
   struct Row
@@ -1952,26 +1958,12 @@ namespace TupErr
   };
 
   static int
-  createDataBase(NDBT_Context* ctx, NDBT_Step* step)
+  populateTable(NDBT_Context* ctx, NDBT_Step* step)
   {
-    // Create table.
-    NDBT_Attribute pk1("pk1", NdbDictionary::Column::Int, 1, true);
-    NDBT_Attribute pk2("pk2", NdbDictionary::Column::Int, 1, true);
-    NDBT_Attribute a1("a1", NdbDictionary::Column::Int, 1);
-  
-    NdbDictionary::Column* columns[] = {&pk1, &pk2, &a1};
-  
-    const NDBT_Table tabDef(tabName, sizeof columns/sizeof columns[0], 
-                            columns);
     Ndb* const ndb = step->getNdb();
-  
-    NdbDictionary::Dictionary* const dictionary = ndb->getDictionary();
-  
-    dictionary->dropTable(tabName);
-    require(dictionary->createTable(tabDef) == 0);
 
     // Populate table.
-    const NdbDictionary::Table* const tab = dictionary->getTable(tabName);
+    const NdbDictionary::Table* const tab = ctx->getTab();
     const NdbRecord* const record = tab->getDefaultRecord();
 
     NdbTransaction* const trans = ndb->startTransaction();
@@ -2014,10 +2006,8 @@ namespace TupErr
     // Build query.
     Ndb* const ndb = step->getNdb();
   
-    NdbDictionary::Dictionary* const dictionary = ndb->getDictionary();
-    const NdbDictionary::Table* const tab = dictionary->getTable(tabName);
+    const NdbDictionary::Table* const tab = ctx->getTab();
     const NdbRecord* const record = tab->getDefaultRecord();
-
   
     NdbTransaction* const trans = ndb->startTransaction();
     if (trans == NULL)
@@ -2072,7 +2062,6 @@ namespace TupErr
       require(false);
     }
     ndb->closeTransaction(trans);
-    dictionary->dropTable(tabName);
 
     return res;
   }
@@ -2083,8 +2072,7 @@ namespace TupErr
     // Build query.
     Ndb* const ndb = step->getNdb();
   
-    NdbDictionary::Dictionary* const dictionary = ndb->getDictionary();
-    const NdbDictionary::Table* const tab = dictionary->getTable(tabName);
+    const NdbDictionary::Table* const tab = ctx->getTab();
     const NdbRecord* const record = tab->getDefaultRecord();
 
     NdbTransaction* const trans = ndb->startTransaction();
@@ -2100,7 +2088,7 @@ namespace TupErr
      * Build an interpreter code sequence that causes rows with pk1==50 to 
      * abort the scan, and that skips all other rows.
      */ 
-    const NdbDictionary::Column* const col = tab->getColumn("pk1");
+    const NdbDictionary::Column* const col = tab->getColumn("PK1");
     require(col != NULL);
     require(code.read_attr(1, col) == 0);
     require(code.load_const_u32(2, 50) == 0);
@@ -2180,7 +2168,6 @@ namespace TupErr
     }
 
     ndb->closeTransaction(trans);
-    dictionary->dropTable(tabName);
 
     return res;
   }
@@ -2863,12 +2850,16 @@ TESTCASE("Bug13394788", "")
   FINALIZER(runClearTable);
 }
 TESTCASE("TupCheckSumError", ""){
-  INITIALIZER(TupErr::createDataBase);
-  INITIALIZER(TupErr::doCheckSumQuery);
+  // TABLE("T18");
+  INITIALIZER(TupErr::populateTable);
+  STEP(TupErr::doCheckSumQuery);
+  FINALIZER(runClearTable);
 }
 TESTCASE("InterpretNok6000", ""){
-  INITIALIZER(TupErr::createDataBase);
-  INITIALIZER(TupErr::doInterpretNok6000Query);
+  // TABLE("T18");
+  INITIALIZER(TupErr::populateTable);
+  STEP(TupErr::doInterpretNok6000Query);
+  FINALIZER(runClearTable);
 }
 TESTCASE("extraNextResultBug11748194",
          "Regression test for bug #11748194")
