@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -257,6 +264,7 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
   if (restarter.waitClusterStarted(120) != 0){
     // If cluster is not started when we shall peform restart
     // the restart can not be executed and the test fails
+    g_err << "Wait cluster start 120 secs failed" << endl;
     return NDBT_FAILED;
   }
   
@@ -272,7 +280,7 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
 	    << endl;
   } else {
     if (restarter.waitClusterStarted(_timeout) != 0){
-      g_err<<"Cluster failed to start" << endl;
+      g_err << "Cluster failed to start, timeout = " << _timeout << endl;
       res = NDBT_FAILED; 
     }
   }
@@ -286,7 +294,11 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
                                 int safety){
   const NdbRestarts::NdbRestart* r = getRestart(_num);
   if (r == NULL)
+  {
+    g_err << "No such restart variant, line: " << __LINE__;
+    g_err << " num: " << _num << endl;
     return NDBT_FAILED;
+  }
 
   int res = executeRestart(ctx, r, _timeout, safety);
   return res;
@@ -298,7 +310,11 @@ int NdbRestarts::executeRestart(NDBT_Context* ctx,
                                 int safety){
   const NdbRestarts::NdbRestart* r = getRestart(_name);
   if (r == NULL)
+  {
+    g_err << "No such restart variant, line: " << __LINE__;
+    g_err << " variant: " << _name << endl;
     return NDBT_FAILED;
+  }
 
   int res = executeRestart(ctx, r, _timeout, safety);
   return res;
