@@ -1,15 +1,21 @@
 /*
-   Copyright (c) 2003-2007 MySQL AB
-   Use is subject to license terms.
+   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -27,14 +33,13 @@
 static int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
                        bool fetch_across_commit, int parallelism=240);
 
-NDB_STD_OPTS_VARS;
-
 const char *load_default_groups[]= { "mysql_cluster",0 };
 
 static const char* _dbname = "TEST_DB";
 static my_bool _transactional = false;
 static my_bool _tupscan = 0;
 static my_bool _diskscan = 0;
+
 static struct my_option my_long_options[] =
 {
   NDB_STD_OPTS("ndb_desc"),
@@ -44,31 +49,29 @@ static struct my_option my_long_options[] =
   { "transactional", 't', "Single transaction (may run out of operations)",
     (uchar**) &_transactional, (uchar**) &_transactional, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "tupscan", 999, "Run tupscan",
+  { "tupscan", NDB_OPT_NOSHORT, "Run tupscan",
     (uchar**) &_tupscan, (uchar**) &_tupscan, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
-  { "diskscan", 999, "Run diskcan",
+  { "diskscan", NDB_OPT_NOSHORT, "Run diskcan",
     (uchar**) &_diskscan, (uchar**) &_diskscan, 0,
     GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0 },
   { 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
+
+static void short_usage_sub(void)
+{
+  ndb_short_usage_sub(NULL);
+}
+
 static void usage()
 {
-#ifdef NOT_USED
-  char desc[] = 
-    "tabname\n"\
-    "This program will delete all records in the specified table using scan delete.\n";
-#endif
-  ndb_std_print_version();
-  print_defaults(MYSQL_CONFIG_NAME,load_default_groups);
-  puts("");
-  my_print_help(my_long_options);
-  my_print_variables(my_long_options);
+  ndb_usage(short_usage_sub, load_default_groups, my_long_options);
 }
 
 int main(int argc, char** argv){
   NDB_INIT(argv[0]);
-  load_defaults("my",load_default_groups,&argc,&argv);
+  ndb_opt_set_usage_funcs(short_usage_sub, usage);
+  ndb_load_defaults(NULL,load_default_groups,&argc,&argv);
   int ho_error;
 #ifndef DBUG_OFF
   opt_debug= "d:t:O,/tmp/ndb_delete_all.trace";
@@ -77,7 +80,7 @@ int main(int argc, char** argv){
 			       ndb_std_get_one_option)))
     return NDBT_ProgramExit(NDBT_WRONGARGS);
 
-  Ndb_cluster_connection con(opt_connect_str);
+  Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   con.set_name("ndb_delete_all");
   if(con.connect(12, 5, 1) != 0)
   {
@@ -92,7 +95,7 @@ int main(int argc, char** argv){
 
   Ndb MyNdb(&con, _dbname );
   if(MyNdb.init() != 0){
-    ERR(MyNdb.getNdbError());
+    NDB_ERR(MyNdb.getNdbError());
     return NDBT_ProgramExit(NDBT_FAILED);
   }
   
@@ -143,7 +146,7 @@ int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
     if (pTrans == NULL) {
       err = pNdb->getNdbError();
       if (err.status == NdbError::TemporaryError){
-	ERR(err);
+	NDB_ERR(err);
 	NdbSleep_MilliSleep(50);
 	continue;
       }
@@ -166,7 +169,7 @@ int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
     if(pTrans->execute(NdbTransaction::NoCommit) != 0){
       err = pTrans->getNdbError();    
       if(err.status == NdbError::TemporaryError){
-	ERR(err);
+	NDB_ERR(err);
 	pNdb->closeTransaction(pTrans);
 	NdbSleep_MilliSleep(50);
 	continue;
@@ -194,7 +197,7 @@ int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
       err = pTrans->getNdbError();    
       if(check == -1){
 	if(err.status == NdbError::TemporaryError){
-	  ERR(err);
+	  NDB_ERR(err);
 	  pNdb->closeTransaction(pTrans);
 	  NdbSleep_MilliSleep(50);
 	  par = 1;
@@ -206,7 +209,7 @@ int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
     if(check == -1){
       err = pTrans->getNdbError();    
       if(err.status == NdbError::TemporaryError){
-	ERR(err);
+	NDB_ERR(err);
 	pNdb->closeTransaction(pTrans);
 	NdbSleep_MilliSleep(50);
 	par = 1;
@@ -226,6 +229,6 @@ int clear_table(Ndb* pNdb, const NdbDictionary::Table* pTab,
   
  failed:
   if(pTrans != 0) pNdb->closeTransaction(pTrans);
-  ERR(err);
+  NDB_ERR(err);
   return (err.code != 0 ? err.code : NDBT_FAILED);
 }
