@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -67,7 +74,9 @@ struct view {
     "  WHEN 3 THEN \"DISCONNECTING\""
     "  ELSE NULL "
     " END AS status, "
-    " remote_address, bytes_sent, bytes_received "
+    " remote_address, bytes_sent, bytes_received, "
+    " connect_count, "
+    " overloaded, overload_count, slowdown, slowdown_count "
     "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>transporters`"
   },
   { "logspaces",
@@ -137,6 +146,12 @@ struct view {
     "  WHEN 23 THEN \"CONST_PRUNED_RANGE_SCANS_RECEIVED\""
     "  WHEN 24 THEN \"LOCAL_READS\""
     "  WHEN 25 THEN \"LOCAL_WRITES\""
+    "  WHEN 26 THEN \"LQHKEY_OVERLOAD\""
+    "  WHEN 27 THEN \"LQHKEY_OVERLOAD_TC\""
+    "  WHEN 28 THEN \"LQHKEY_OVERLOAD_READER\""
+    "  WHEN 29 THEN \"LQHKEY_OVERLOAD_NODE_PEER\""
+    "  WHEN 30 THEN \"LQHKEY_OVERLOAD_SUBSCRIBER\""
+    "  WHEN 31 THEN \"LQHSCAN_SLOWDOWNS\""
     "  ELSE \"<unknown>\" "
     " END AS counter_name, "
     "val "
@@ -254,6 +269,65 @@ struct view {
     "FROM `<NDBINFO_DB>`.cluster_operations o "
     "JOIN information_schema.ndb_transid_mysql_connection_map map"
     "  ON (map.ndb_transid >> 32) = (o.transid >> 32)"
+  },
+  { "membership",
+    "SELECT node_id, group_id, left_node, right_node, president, successor, "
+    "dynamic_id & 0xFFFF AS succession_order, "
+    "dynamic_id >> 16 AS Conf_HB_order, "
+    "arbitrator, arb_ticket, "
+    "CASE arb_state"
+    "  WHEN 0 THEN \"ARBIT_NULL\""
+    "  WHEN 1 THEN \"ARBIT_INIT\""
+    "  WHEN 2 THEN \"ARBIT_FIND\""
+    "  WHEN 3 THEN \"ARBIT_PREP1\""
+    "  WHEN 4 THEN \"ARBIT_PREP2\""
+    "  WHEN 5 THEN \"ARBIT_START\""
+    "  WHEN 6 THEN \"ARBIT_RUN\""
+    "  WHEN 7 THEN \"ARBIT_CHOOSE\""
+    "  WHEN 8 THEN \"ARBIT_CRASH\""
+    "  ELSE \"UNKNOWN\""
+    " END AS arb_state, "
+    "CASE arb_connected"
+    "  WHEN 1 THEN \"Yes\""
+    "  ELSE \"No\""
+    " END AS arb_connected, "
+    "conn_rank1_arbs AS connected_rank1_arbs, "
+    "conn_rank2_arbs AS connected_rank2_arbs "
+    "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>membership`"
+  },
+  { "arbitrator_validity_detail",
+    "SELECT node_id, "
+    "arbitrator, "
+    "arb_ticket, "
+    "CASE arb_connected"
+    "  WHEN 1 THEN \"Yes\""
+    "  ELSE \"No\""
+    " END AS arb_connected, "
+    "CASE arb_state"
+    "  WHEN 0 THEN \"ARBIT_NULL\""
+    "  WHEN 1 THEN \"ARBIT_INIT\""
+    "  WHEN 2 THEN \"ARBIT_FIND\""
+    "  WHEN 3 THEN \"ARBIT_PREP1\""
+    "  WHEN 4 THEN \"ARBIT_PREP2\""
+    "  WHEN 5 THEN \"ARBIT_START\""
+    "  WHEN 6 THEN \"ARBIT_RUN\""
+    "  WHEN 7 THEN \"ARBIT_CHOOSE\""
+    "  WHEN 8 THEN \"ARBIT_CRASH\""
+    "  ELSE \"UNKNOWN\""
+    " END AS arb_state "
+    "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>membership` "
+    "ORDER BY arbitrator, arb_connected DESC"
+  },
+  { "arbitrator_validity_summary",
+    "SELECT arbitrator, "
+    "arb_ticket, "
+    "CASE arb_connected"
+    "  WHEN 1 THEN \"Yes\""
+    "  ELSE \"No\""
+    " END AS arb_connected, "
+    "count(*) as consensus_count "
+    "FROM `<NDBINFO_DB>`.`<TABLE_PREFIX>membership` "
+    "GROUP BY arbitrator, arb_ticket, arb_connected"
   }
 };
 
@@ -439,19 +513,6 @@ int main(int argc, char** argv){
              " (@@ndbinfo_version >= (7 << 16) | (1 << 8)) || @ndbinfo_skip_version_check");
   print_conditional_sql(sql);
 
-  printf("# Only create objects if ndbinfo namespace is free\n");
-  sql.assfmt("SET @@ndbinfo_show_hidden=TRUE");
-  print_conditional_sql(sql);
-  sql.assfmt("SELECT @have_ndbinfo:= COUNT(*) = 0"
-             " FROM information_schema.tables WHERE"
-             " table_schema = @@ndbinfo_database AND"
-             " LEFT(table_name, LENGTH(@@ndbinfo_table_prefix)) ="
-             " @@ndbinfo_table_prefix AND"
-             " engine != \"ndbinfo\"");
-  print_conditional_sql(sql);
-  sql.assfmt("SET @@ndbinfo_show_hidden=default");
-  print_conditional_sql(sql);
-
   sql.assfmt("CREATE DATABASE IF NOT EXISTS `%s`", opt_ndbinfo_db);
   print_conditional_sql(sql);
 
@@ -561,7 +622,7 @@ int main(int argc, char** argv){
 
     /* Create or replace the view */
     BaseString sql;
-    sql.assfmt("CREATE OR REPLACE DEFINER=`root@localhost` "
+    sql.assfmt("CREATE OR REPLACE DEFINER=`root`@`localhost` "
                "SQL SECURITY INVOKER VIEW `%s`.`%s` AS %s",
                opt_ndbinfo_db, v.name, view_sql.c_str());
     print_conditional_sql(sql);
