@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -277,6 +284,11 @@ RestoreMetaData::getTable(Uint32 tableId) const {
   return NULL;
 }
 
+Uint32
+RestoreMetaData::getStartGCP() const {
+  return m_startGCP;
+}
+
 Uint32
 RestoreMetaData::getStopGCP() const {
   return m_stopGCP;
@@ -466,6 +478,19 @@ RestoreMetaData::readMetaTableDesc() {
     return true;
     break;
   }
+  case DictTabInfo::ForeignKey:
+  {
+    NdbDictionary::ForeignKey * dst = new NdbDictionary::ForeignKey;
+    errcode =
+      NdbDictInterface::parseForeignKeyInfo(NdbForeignKeyImpl::getImpl(* dst),
+                                            (const Uint32*)ptr, len);
+    if (errcode)
+      delete dst;
+    obj.m_objPtr = dst;
+    debug << hex << obj.m_objPtr << " " 
+	   << dec << dst->getObjectId() << " " << dst->getName() << endl;
+    break;
+  }
   default:
     if (ga_skip_unknown_objects)
     {
@@ -671,7 +696,12 @@ RestoreMetaData::readGCPEntry() {
   dst.StopGCP = ntohl(dst.StopGCP);
   
   m_startGCP = dst.StartGCP;
-  m_stopGCP = dst.StopGCP;
+  /**
+   * Stop GCP is recorded as StopGCP -1 by Backup.cpp
+   * We correct this here
+   * Backup format not changed
+   */
+  m_stopGCP = dst.StopGCP + 1;
   return true;
 }
 
@@ -1941,12 +1971,14 @@ RestoreLogIterator::RestoreLogIterator(const RestoreMetaData & md)
 const LogEntry *
 RestoreLogIterator::getNextLogEntry(int & res) {
   // Read record length
+  const Uint32 startGCP = m_metaData.getStartGCP();
   const Uint32 stopGCP = m_metaData.getStopGCP();
   Uint32 tableId;
   Uint32 triggerEvent;
   Uint32 frag_id;
   Uint32 *attr_data;
   Uint32 attr_data_len;
+  bool skip_entry = false;
   do {
     Uint32 len;
     Uint32 *logEntryPtr;
@@ -2019,7 +2051,20 @@ RestoreLogIterator::getNextLogEntry(int & res) {
       attr_data_len--;
       m_last_gci = ntohl(*(attr_data + attr_data_len));
     }
-  } while(m_last_gci > stopGCP + 1);
+    if (m_is_undolog)
+    {
+      // Do not apply anything from startGCP or lower
+      skip_entry = (m_last_gci <= startGCP);
+    }
+    else
+    {
+      // Do not apply anything after stopGCP
+      skip_entry = (m_last_gci > stopGCP);
+    }
+    // Skip entries instead of stopping scan since entries are not ordered
+    // by GCP. Entries from different GCPs may be interleaved, so scan till
+    // EOF to read all matching entries.
+  } while (skip_entry);
 
   m_logEntry.m_table = m_metaData.getTable(tableId);
   /* We should 'invert' the operation type when we restore an Undo log.
@@ -2157,12 +2202,11 @@ operator<<(NdbOut& ndbout, const LogEntry& logE)
   return ndbout;
 }
 
-#include <NDBT.hpp>
-
-NdbOut & 
-operator<<(NdbOut& ndbout, const TableS & table){
-  
-  ndbout << (* (NDBT_Table*)table.m_dictTable) << endl;
+NdbOut &
+operator<<(NdbOut& ndbout, const TableS & table)
+{
+  ndbout << "-- " << table.getTableName() << " --" << endl;
+  ndbout << *(table.m_dictTable) << endl;
   return ndbout;
 }
 
