@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2019, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -431,6 +438,7 @@ public:
   Uint32 getObjType(Uint32 i) const { return m_objects[i].m_objType; }
   void* getObjPtr(Uint32 i) const { return m_objects[i].m_objPtr; }
   
+  Uint32 getStartGCP() const;
   Uint32 getStopGCP() const;
   Uint32 getNdbVersion() const { return m_fileHeader.NdbVersion; };
 }; // RestoreMetaData
@@ -533,6 +541,10 @@ public:
   RestoreLogIterator(const RestoreMetaData &);
   virtual ~RestoreLogIterator() {};
 
+  bool isSnapshotstartBackup()
+  {
+    return m_is_undolog;
+  }
   const LogEntry * getNextLogEntry(int & res);
 };
 
